@@ -17,11 +17,33 @@
 /* Define if you have the Andrew File System.  */
 #undef AFS
 
-/* Define if your system has the ftruncate function or if it can
-   emulate it.  */
-#undef HAVE_FTRUNCATE
+/* Define to 1 if NLS is requested.  */
+#undef ENABLE_NLS
 
+/* Define if your system defines TIOCGWINSZ in sys/ioctl.h.  */
+#undef GWINSZ_IN_SYS_IOCTL
 
+/* Define as 1 if you have catgets and don't want to use GNU gettext.  */
+#undef HAVE_CATGETS
+
+/* Define as 1 if you have gettext and don't want to use GNU gettext.  */
+#undef HAVE_GETTEXT
+
+/* Define if your locale.h file contains LC_MESSAGES.  */
+#undef HAVE_LC_MESSAGES
+
+/* Define to 1 if you have the stpcpy function.  */
+#undef HAVE_STPCPY
+
+/* Define to `unsigned long' if <sys/types.h> doesn't define.  */
+#undef ino_t
+
+/* Define if there is a member named d_ino in the struct describing
+   directory headers.  */
+#undef D_INO_IN_DIRENT
+
+/* Define if `struct utimbuf' is declared -- usually in <utime.h>.  */
+#undef HAVE_STRUCT_UTIMBUF
 
 
 /* Define one of the following to indicate how a program can
@@ -57,12 +79,24 @@
    of mounted filesystems.  [4.4BSD]  */
 #undef MOUNTED_GETMNTINFO
 
+/* Define if there is a function named listmntent that can be used to
+   list all mounted filesystems. [UNICOS] */
+#undef MOUNTED_LISTMNTENT
+
 /* Define if there is a function named mntctl that can be used to read
    the list of mounted filesystems, and there is a system header file
    that declares `struct vmount.'  [AIX]  */
 #undef MOUNTED_VMOUNT
 
+/* Define to the name of the distribution.  */
+#undef PACKAGE
 
+/* The concatenation of the strings PACKAGE, "-", and VERSION.  */
+#undef PACKAGE_VERSION
+
+
+/* Define to 1 if ANSI function prototypes are usable.  */
+#undef PROTOTYPES
 
 
 /* Define one of the following to indicate how a program can obtain
@@ -72,15 +106,15 @@
 #undef STAT_STATFS3_OSF1
 
 /* Define if there is no specific function for reading filesystems usage
-   information;  Use `read'.  [SVR2]  */
-#undef STAT_READ
+   information and you have the <sys/filsys.h> header file.  [SVR2]  */
+#undef STAT_READ_FILSYS
 
 /* Define if statfs takes 2 args and struct statfs has a field named f_bsize.
    [4.3BSD, SunOS 4, HP-UX, AIX PS/2]  */
 #undef STAT_STATFS2_BSIZE
 
 /* Define if statfs takes 2 args and struct statfs has a field named f_fsize.
-   [4.4BSD]  */
+   [4.4BSD, NetBSD]  */
 #undef STAT_STATFS2_FSIZE
 
 /* Define if statfs takes 2 args and the second argument has
@@ -93,6 +127,15 @@
 /* Define if there is a function named statvfs.  [SVR4]  */
 #undef STAT_STATVFS
 
+/* Define if the block counts reported by statfs may be truncated to 2GB
+   and the correct values may be stored in the f_spare array.
+   [SunOS 4.1.2, 4.1.3, and 4.1.3_U1 are reported to have this problem.
+   SunOS 4.1.1 seems not to be affected.]  */
+#undef STATFS_TRUNCATES_BLOCK_COUNTS
+
+/* Define to the version of the distribution.  */
+#undef VERSION
+
 
 /* Leave that blank line there!!  Autoheader needs it.
    If you're adding to this file, keep in mind:
