@@ -19,68 +19,56 @@
    Some algorithms adapted from GNU Emacs. */
 
 #ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
-#include <config.h>
-#else
-#include "config.h"
-#endif
+# include <config.h>
 #endif
 
 #include <stdio.h>
 #include <ctype.h>
 #include <sys/types.h>
 #include "backupfile.h"
-#if defined(STDC_HEADERS) || defined(HAVE_STRING_H)
-#include <string.h>
-#ifndef index
-#define index strchr
-#endif
-#ifndef rindex
-#define rindex strrchr
-#endif
+#ifdef HAVE_STRING_H
+# include <string.h>
 #else
-#include <strings.h>
+# include <strings.h>
 #endif
 
-#if defined(DIRENT) || defined(_POSIX_VERSION)
-#include <dirent.h>
-#define NLENGTH(direct) (strlen((direct)->d_name))
-#else /* not (DIRENT or _POSIX_VERSION) */
-#define dirent direct
-#define NLENGTH(direct) ((direct)->d_namlen)
-#ifdef SYSNDIR
-#include <sys/ndir.h>
-#endif /* SYSNDIR */
-#ifdef SYSDIR
-#include <sys/dir.h>
-#endif /* SYSDIR */
-#ifdef NDIR
-#include <ndir.h>
-#endif /* NDIR */
-#endif /* DIRENT or _POSIX_VERSION */
+#ifdef HAVE_DIRENT_H
+# include <dirent.h>
+# define NLENGTH(direct) (strlen((direct)->d_name))
+#else /* not HAVE_DIRENT_H */
+# define dirent direct
+# define NLENGTH(direct) ((direct)->d_namlen)
+# ifdef HAVE_SYS_NDIR_H
+#  include <sys/ndir.h>
+# endif /* HAVE_SYS_NDIR_H */
+# ifdef HAVE_SYS_DIR_H
+#  include <sys/dir.h>
+# endif /* HAVE_SYS_DIR_H */
+# ifdef HAVE_NDIR_H
+#  include <ndir.h>
+# endif /* HAVE_NDIR_H */
+#endif /* HAVE_DIRENT_H */
 
-#ifdef VOID_CLOSEDIR
+#ifdef CLOSEDIR_VOID
 /* Fake a return value. */
-#define CLOSEDIR(d) (closedir (d), 0)
+# define CLOSEDIR(d) (closedir (d), 0)
 #else
-#define CLOSEDIR(d) closedir (d)
+# define CLOSEDIR(d) closedir (d)
 #endif
 
 #ifdef STDC_HEADERS
-#include <stdlib.h>
+# include <stdlib.h>
 #else
 char *malloc ();
 #endif
 
-#if !defined (isascii) || defined (STDC_HEADERS)
-#undef isascii
-#define isascii(c) 1
+#if defined (STDC_HEADERS) || (!defined (isascii) && !defined (HAVE_ISASCII))
+# define IN_CTYPE_DOMAIN(c) 1
+#else
+# define IN_CTYPE_DOMAIN(c) isascii(c)
 #endif
 
-#define ISDIGIT(c) (isascii ((unsigned char ) c) \
+#define ISDIGIT(c) (IN_CTYPE_DOMAIN ((unsigned char) (c)) \
 		    && isdigit ((unsigned char) (c)))
 
 #if defined (HAVE_UNISTD_H)
@@ -90,9 +78,9 @@ char *malloc ();
 #if defined (_POSIX_VERSION)
 /* POSIX does not require that the d_ino field be present, and some
    systems do not provide it. */
-#define REAL_DIR_ENTRY(dp) 1
+# define REAL_DIR_ENTRY(dp) 1
 #else
-#define REAL_DIR_ENTRY(dp) ((dp)->d_ino != 0)
+# define REAL_DIR_ENTRY(dp) ((dp)->d_ino != 0)
 #endif
 
 /* Which type of backup file names are generated. */
@@ -117,7 +105,7 @@ static int version_number ();
 
 char *
 find_backup_file_name (file)
-     char *file;
+     const char *file;
 {
   char *dir;
   char *base_versions;
@@ -149,13 +137,14 @@ find_backup_file_name (file)
 
 static int
 max_backup_version (file, dir)
-     char *file, *dir;
+     const char *file;
+     const char *dir;
 {
   DIR *dirp;
   struct dirent *dp;
   int highest_version;
   int this_version;
-  int file_name_length;
+  size_t file_name_length;
 
   dirp = opendir (dir);
   if (!dirp)
@@ -183,7 +172,7 @@ max_backup_version (file, dir)
 
 static char *
 make_version_name (file, version)
-     char *file;
+     const char *file;
      int version;
 {
   char *backup_name;
@@ -201,12 +190,12 @@ make_version_name (file, version)
 
 static int
 version_number (base, backup, base_length)
-     char *base;
-     char *backup;
+     const char *base;
+     const char *backup;
      int base_length;
 {
   int version;
-  char *p;
+  const char *p;
 
   version = 0;
   if (!strncmp (base, backup, base_length) && ISDIGIT (backup[base_length]))
@@ -224,7 +213,8 @@ version_number (base, backup, base_length)
 
 static char *
 concat (str1, str2)
-     char *str1, *str2;
+     const char *str1;
+     const char *str2;
 {
   char *newstr;
   int str1_length = strlen (str1);
