@@ -16,33 +16,23 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
 #endif
 
-#if defined(STDC_HEADERS) || defined(HAVE_STRING_H)
-#include <string.h>
-#ifndef rindex
-#define rindex strrchr
-#endif
-#else
-#include <strings.h>
-#endif
-
-/* Return NAME with any leading path stripped off.  */
+/* Return NAME with any leading path stripped off.
+   Don't use strrchr/rindex.  */
 
 char *
 basename (name)
-     char *name;
+     const char *name;
 {
-  char *base;
+  const char *base = name;
 
-  base = rindex (name, '/');
-  return base ? base + 1 : name;
+  while (*name)
+    {
+      if (*name == '/')
+	base = name + 1;
+      ++name;
+    }
+  return (char *) base;
 }
