@@ -22,7 +22,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "fsusage.h"
-#include "safe-stat.h"
 
 int statfs ();
 
@@ -38,7 +37,7 @@ int statfs ();
 #include <sys/vfs.h>
 #endif
 
-#ifdef HAVE_SYS_FILSYS_H
+#if defined(HAVE_SYS_FILSYS_H) && !defined(_CRAY)
 #include <sys/filsys.h>		/* SVR2.  */
 #endif
 
@@ -59,6 +58,8 @@ int statfs ();
 int statvfs ();
 #endif
 
+int safe_read ();
+
 /* Return the number of TOSIZE-byte blocks used by
    BLOCKS FROMSIZE-byte blocks, rounding away from zero.
    TOSIZE must be positive.  Return -1 if FROMSIZE is not positive.  */
@@ -72,7 +73,7 @@ adjust_blocks (blocks, fromsize, tosize)
     abort ();
   if (fromsize <= 0)
     return -1;
-								    
+
   if (fromsize == tosize)	/* E.g., from 512 to 512.  */
     return blocks;
   else if (fromsize > tosize)	/* E.g., from 2048 to 512.  */
@@ -89,7 +90,8 @@ adjust_blocks (blocks, fromsize, tosize)
 
 int
 get_fs_usage (path, disk, fsp)
-     char *path, *disk;
+     const char *path;
+     const char *disk;
      struct fs_usage *fsp;
 {
 #if defined (STAT_STATFS3_OSF1)
@@ -124,7 +126,7 @@ get_fs_usage (path, disk, fsp)
   if (fd < 0)
     return -1;
   lseek (fd, (long) SUPERBOFF, 0);
-  if (read (fd, (char *) &fsd, sizeof fsd) != sizeof fsd)
+  if (safe_read (fd, (char *) &fsd, sizeof fsd) != sizeof fsd)
     {
       close (fd);
       return -1;
@@ -143,6 +145,21 @@ get_fs_usage (path, disk, fsp)
 
   if (statfs (path, &fsd) < 0)
     return -1;
+
+#ifdef STATFS_TRUNCATES_BLOCK_COUNTS
+  /* In SunOS 4.1.2, 4.1.3, and 4.1.3_U1, the block counts in the
+     struct statfs are truncated to 2GB.  These conditions detect that
+     truncation, presumably without botching the 4.1.1 case, in which
+     the values are not truncated.  The correct counts are stored in
+     undocumented spare fields.  */
+  if (fsd.f_blocks == 0x1fffff && fsd.f_spare[0] > 0)
+    {
+      fsd.f_blocks = fsd.f_spare[0];
+      fsd.f_bfree = fsd.f_spare[1];
+      fsd.f_bavail = fsd.f_spare[2];
+    }
+#endif /* STATFS_TRUNCATES_BLOCK_COUNTS */
+
 #define CONVERT_BLOCKS(b) adjust_blocks ((b), fsd.f_bsize, 512)
 #endif
 
@@ -206,7 +223,7 @@ statfs (path, fsb)
   struct stat stats;
   struct dustat fsd;
 
-  if (SAFE_STAT (path, &stats))
+  if (stat (path, &stats))
     return -1;
   if (dustat (stats.st_dev, 0, &fsd, sizeof (fsd)))
     return -1;
