@@ -1,5 +1,5 @@
 /* chgrp -- change group ownership of files
-   Copyright (C) 1989, 1990, 1991 Free Software Foundation, Inc.
+   Copyright (C) 89, 90, 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,51 +12,66 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Written by David MacKenzie <djm@gnu.ai.mit.edu>. */
 
+#include <config.h>
 #include <stdio.h>
-#include <ctype.h>
 #include <sys/types.h>
 #include <grp.h>
 #include <getopt.h>
-#include "system.h"
-#include "version.h"
 
-#if !defined (isascii) || defined (STDC_HEADERS)
-#undef isascii
-#define isascii(c) 1
+#if HAVE_LIMITS_H
+# include <limits.h>
 #endif
 
-#define ISDIGIT(c) (isascii (c) && isdigit (c))
+#ifndef UINT_MAX
+# define UINT_MAX ((unsigned int) ~(unsigned int) 0)
+#endif
+
+#ifndef INT_MAX
+# define INT_MAX ((int) (UINT_MAX >> 1))
+#endif
+
+#include "system.h"
+#include "xstrtoul.h"
+#include "error.h"
+
+/* MAXUID may come from limits.h *or* sys/params.h (via system.h) above. */
+#ifndef MAXUID
+# define MAXUID INT_MAX
+#endif
 
 #ifndef _POSIX_VERSION
 struct group *getgrnam ();
 #endif
 
-#ifdef _POSIX_SOURCE
-#define endgrent()
+#ifndef HAVE_ENDGRENT
+# define endgrent() ((void) 0)
 #endif
 
-int lstat ();
+#ifdef HAVE_LCHOWN
+# define LCHOWN(FILE, OWNER, GROUP) lchown (FILE, OWNER, GROUP)
+#else
+# define LCHOWN(FILE, OWNER, GROUP) 1
+#endif
 
+char *group_member ();
 char *savedir ();
 char *xmalloc ();
 char *xrealloc ();
-void error ();
 
-static int change_file_group ();
-static int change_dir_group ();
-static int isnumber ();
-static void describe_change ();
-static void parse_group ();
-static void usage ();
+static int change_dir_group __P ((char *dir, int group, struct stat *statp));
 
 /* The name the program was run with. */
 char *program_name;
 
+/* If nonzero, and the systems has support for it, change the ownership
+   of symbolic links rather than any files they point to.  */
+static int change_symlinks;
+
 /* If nonzero, change the ownership of directories recursively. */
 static int recurse;
 
@@ -72,100 +87,60 @@ static int changes_only;
 /* The name of the group to which ownership of the files is being given. */
 static char *groupname;
 
-/* If non-zero, display usage information and exit.  */
-static int flag_help;
+/* If nonzero, display usage information and exit.  */
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If nonzero, print the version on standard output and exit.  */
+static int show_version;
 
 static struct option const long_options[] =
 {
   {"recursive", no_argument, 0, 'R'},
   {"changes", no_argument, 0, 'c'},
+  {"no-dereference", no_argument, 0, 'h'},
   {"silent", no_argument, 0, 'f'},
   {"quiet", no_argument, 0, 'f'},
   {"verbose", no_argument, 0, 'v'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {0, 0, 0, 0}
 };
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+/* Tell the user the group name to which ownership of FILE
+   has been given; if CHANGED is zero, FILE was that group already. */
+
+static void
+describe_change (char *file, int changed)
 {
-  int group;
-  int errors = 0;
-  int optc;
-
-  program_name = argv[0];
-  recurse = force_silent = verbose = changes_only = 0;
-
-  while ((optc = getopt_long (argc, argv, "Rcfv", long_options, (int *) 0))
-	 != EOF)
-    {
-      switch (optc)
-	{
-	case 0:
-	  break;
-	case 'R':
-	  recurse = 1;
-	  break;
-	case 'c':
-	  verbose = 1;
-	  changes_only = 1;
-	  break;
-	case 'f':
-	  force_silent = 1;
-	  break;
-	case 'v':
-	  verbose = 1;
-	  break;
-	default:
-	  usage ();
-	}
-    }
-
-  if (flag_version)
-    {
-      fprintf (stderr, "%s\n", version_string);
-      exit (0);
-    }
-
-  if (flag_help)
-    usage ();
-
-  if (optind >= argc - 1)
-    usage ();
-
-  parse_group (argv[optind++], &group);
-
-  for (; optind < argc; ++optind)
-    errors |= change_file_group (argv[optind], group);
-
-  exit (errors);
+  if (changed)
+    printf (_("group of %s changed to %s\n"), file, groupname);
+  else
+    printf (_("group of %s retained as %s\n"), file, groupname);
 }
 
 /* Set *G according to NAME. */
 
 static void
-parse_group (name, g)
-     char *name;
-     int *g;
+parse_group (char *name, int *g)
 {
   struct group *grp;
 
   groupname = name;
   if (*name == '\0')
-    error (1, 0, "can not change to null group");
+    error (1, 0, _("can not change to null group"));
 
   grp = getgrnam (name);
   if (grp == NULL)
     {
-      if (!isnumber (name))
-	error (1, 0, "invalid group `%s'", name);
-      *g = atoi (name);
+      strtol_error s_err;
+      unsigned long int tmp_long;
+
+      s_err = xstrtoul (name, NULL, 0, &tmp_long, NULL);
+      *g = tmp_long;
+      if (s_err == LONGINT_OVERFLOW || tmp_long > INT_MAX)
+	{
+	  STRTOL_FATAL_ERROR (name, _("group number"), s_err);
+	}
     }
   else
     *g = grp->gr_gid;
@@ -177,9 +152,7 @@ parse_group (name, g)
    Return 0 if successful, 1 if errors occurred. */
 
 static int
-change_file_group (file, group)
-     char *file;
-     int group;
+change_file_group (char *file, int group)
 {
   struct stat file_stats;
   int errors = 0;
@@ -193,13 +166,38 @@ change_file_group (file, group)
 
   if (group != file_stats.st_gid)
     {
+      int fail;
+
       if (verbose)
 	describe_change (file, 1);
-      if (chown (file, file_stats.st_uid, group))
+
+      if (change_symlinks)
+	fail = LCHOWN (file, file_stats.st_uid, group);
+      else
+	fail = chown (file, file_stats.st_uid, group);
+
+      if (fail)
 	{
-	  if (force_silent == 0)
-	    error (0, errno, "%s", file);
 	  errors = 1;
+	  if (force_silent == 0)
+	    {
+	      /* Give a more specific message.  Some systems set errno
+		 to EPERM for both `inaccessible file' and `user not a member
+		 of the specified group' errors.  */
+	      if (errno == EPERM && !group_member (group))
+		{
+		  error (0, errno, _("you are not a member of group `%s'"),
+			 groupname);
+		}
+	      else if (errno == EINVAL && group > MAXUID)
+		{
+		  error (0, 0, _("%s: invalid group number"), groupname);
+		}
+	      else
+		{
+		  error (0, errno, "%s", file);
+		}
+	    }
 	}
     }
   else if (verbose && changes_only == 0)
@@ -216,10 +214,7 @@ change_file_group (file, group)
    Return 0 if successful, 1 if errors occurred. */
 
 static int
-change_dir_group (dir, group, statp)
-     char *dir;
-     int group;
-     struct stat *statp;
+change_dir_group (char *dir, int group, struct stat *statp)
 {
   char *name_space, *namep;
   char *path;			/* Full path of each entry to process. */
@@ -239,7 +234,7 @@ change_dir_group (dir, group, statp)
 	  return 1;
 	}
       else
-	error (1, 0, "virtual memory exhausted");
+	error (1, 0, _("virtual memory exhausted"));
     }
 
   dirlength = strlen (dir) + 1;	/* + 1 is for the trailing '/'. */
@@ -265,39 +260,94 @@ change_dir_group (dir, group, statp)
   return errors;
 }
 
-/* Tell the user the group name to which ownership of FILE
-   has been given; if CHANGED is zero, FILE was that group already. */
-
 static void
-describe_change (file, changed)
-     char *file;
-     int changed;
+usage (int status)
 {
-  if (changed)
-    printf ("group of %s changed to %s\n", file, groupname);
+  if (status != 0)
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
+	     program_name);
   else
-    printf ("group of %s retained as %s\n", file, groupname);
+    {
+      printf (_("Usage: %s [OPTION]... GROUP FILE...\n"), program_name);
+      printf (_("\
+Change the group membership of each FILE to GROUP.\n\
+\n\
+  -c, --changes           like verbose but report only when a change is made\n\
+  -h, --no-dereference    affect symbolic links instead of any referenced file\n\
+                          (available only on systems with lchown system call)\n\
+  -f, --silent, --quiet   suppress most error messages\n\
+  -v, --verbose           output a diagnostic for every file processed\n\
+  -R, --recursive         change files and directories recursively\n\
+      --help              display this help and exit\n\
+      --version           output version information and exit\n"));
+    }
+  exit (status);
 }
 
-/* Return nonzero if STR represents an unsigned decimal integer,
-   otherwise return 0. */
-
-static int
-isnumber (str)
-     char *str;
+int
+main (int argc, char **argv)
 {
-  for (; *str; str++)
-    if (!ISDIGIT (*str))
-      return 0;
-  return 1;
-}
+  int group;
+  int errors = 0;
+  int optc;
 
-static void
-usage ()
-{
-  fprintf (stderr, "\
-Usage: %s [-Rcfv] [--recursive] [--changes] [--silent] [--quiet]\n\
-       [--verbose] [--help] [--version] group file...\n",
-	   program_name);
-  exit (1);
+  program_name = argv[0];
+  recurse = force_silent = verbose = changes_only = 0;
+
+  while ((optc = getopt_long (argc, argv, "Rcfnv", long_options, (int *) 0))
+	 != EOF)
+    {
+      switch (optc)
+	{
+	case 0:
+	  break;
+	case 'R':
+	  recurse = 1;
+	  break;
+	case 'c':
+	  verbose = 1;
+	  changes_only = 1;
+	  break;
+	case 'f':
+	  force_silent = 1;
+	  break;
+	case 'h':
+	  change_symlinks = 1;
+	  break;
+	case 'v':
+	  verbose = 1;
+	  break;
+	default:
+	  usage (1);
+	}
+    }
+
+  if (show_version)
+    {
+      printf ("chgrp - %s\n", PACKAGE_VERSION);
+      exit (0);
+    }
+
+  if (show_help)
+    usage (0);
+
+  if (argc - optind <= 1)
+    {
+      error (0, 0, _("too few arguments"));
+      usage (1);
+    }
+
+#ifndef HAVE_LCHOWN
+  if (change_symlinks)
+    {
+      error (1, 0, _("--no-dereference (-h) is not supported on this system"));
+    }
+#endif
+
+  parse_group (argv[optind++], &group);
+
+  for (; optind < argc; ++optind)
+    errors |= change_file_group (argv[optind], group);
+
+  exit (errors);
 }
