@@ -1,5 +1,5 @@
 /* cp-hash.c  -- file copying (hash search routines)
-   Copyright (C) 1989, 1990, 1991 Free Software Foundation.
+   Copyright (C) 89, 90, 91, 95, 1996 Free Software Foundation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,18 +12,14 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Written by Torbjorn Granlund, Sweden (tege@sics.se). */
 
 #include <config.h>
 #include <stdio.h>
 #include "cp.h"
-#include "safe-stat.h"
-
-char *hash_insert ();
-char *hash_insert2 ();
 
 struct htab *htab;
 char new_file;
@@ -32,12 +28,11 @@ char new_file;
    Return 0 if successful, 1 if not. */
 
 int
-remember_created (path)
-     char *path;
+remember_created (const char *path)
 {
   struct stat sb;
 
-  if (SAFE_STAT (path, &sb) < 0)
+  if (stat (path, &sb) < 0)
     {
       error (0, errno, "%s", path);
       return 1;
@@ -52,10 +47,7 @@ remember_created (path)
    Return NULL if inserted, otherwise non-NULL. */
 
 char *
-remember_copied (node, ino, dev)
-     char *node;
-     ino_t ino;
-     dev_t dev;
+remember_copied (const char *node, ino_t ino, dev_t dev)
 {
   return hash_insert (ino, dev, node);
 }
@@ -68,9 +60,7 @@ remember_copied (node, ino, dev)
    doubled.)  */
 
 void
-hash_init (modulus, entry_tab_size)
-     unsigned modulus;
-     unsigned entry_tab_size;
+hash_init (unsigned int modulus, unsigned int entry_tab_size)
 {
   struct htab *htab_r;
 
@@ -91,7 +81,7 @@ hash_init (modulus, entry_tab_size)
    contain no entries.  */
 
 void
-forget_all ()
+forget_all (void)
 {
   int i;
   struct entry **p;
@@ -109,10 +99,7 @@ forget_all ()
    Return NULL if inserted, otherwise non-NULL. */
 
 char *
-hash_insert (ino, dev, node)
-     ino_t ino;
-     dev_t dev;
-     char *node;
+hash_insert (ino_t ino, dev_t dev, const char *node)
 {
   struct htab *htab_r = htab;
 
@@ -179,11 +166,7 @@ hash_insert (ino, dev, node)
    Return NULL if inserted, otherwise non-NULL. */
 
 char *
-hash_insert2 (htab, ino, dev, node)
-     struct htab *htab;
-     ino_t ino;
-     dev_t dev;
-     char *node;
+hash_insert2 (struct htab *htab, ino_t ino, dev_t dev, const char *node)
 {
   struct entry **hp, *ep2, *ep;
   hp = &htab->hash[ino % htab->modulus];
@@ -212,7 +195,7 @@ hash_insert2 (htab, ino, dev, node)
   ep = *hp = &htab->entry_tab[htab->first_free_entry++];
   ep->ino = ino;
   ep->dev = dev;
-  ep->node = node;
+  ep->node = (char *) node;
   ep->coll_link = ep2;		/* ep2 is NULL if not collision.  */
 
   return NULL;
