@@ -1,5 +1,5 @@
 /* cp.c  -- file copying (main routines)
-   Copyright (C) 1989, 1990, 1991 Free Software Foundation.
+   Copyright (C) 89, 90, 91, 95, 1996 Free Software Foundation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Written by Torbjorn Granlund, David MacKenzie, and Jim Meyering. */
 
@@ -23,17 +23,23 @@
 
 #include <config.h>
 #include <stdio.h>
+
+#define NDEBUG
+#include <assert.h>
+
 #include <getopt.h>
 #include "cp.h"
 #include "backupfile.h"
-#include "version.h"
-#include "safe-stat.h"
-#include "safe-lstat.h"
+#include "argmatch.h"
 
 #ifndef _POSIX_VERSION
 uid_t geteuid ();
 #endif
 
+#ifdef HAVE_LCHOWN
+# define chown(PATH, OWNER, GROUP) lchown(PATH, OWNER, GROUP)
+#endif
+
 /* Used by do_copy, make_path_private, and re_protect
    to keep a list of leading directories whose protections
    need to be fixed after copying. */
@@ -44,18 +50,46 @@ struct dir_attr
   struct dir_attr *next;
 };
 
+/* Control creation of sparse files (files with holes).  */
+enum Sparse_type
+{
+  /* Never create holes in DEST.  */
+  SPARSE_NEVER,
+
+  /* This is the default.  Use a crude (and sometimes inaccurate)
+     heuristic to determine if SOURCE has holes.  If so, try to create
+     holes in DEST.  */
+  SPARSE_AUTO,
+
+  /* For every sufficiently long sequence of bytes in SOURCE, try to
+     create a corresponding hole in DEST.  There is a performance penalty
+     here because CP has to search for holes in SRC.  But if the holes are
+     big enough, that penalty can be offset by the decrease in the amount
+     of data written to disk.   */
+  SPARSE_ALWAYS
+};
+
+int stat ();
+int lstat ();
+
 char *dirname ();
 char *xstrdup ();
 enum backup_type get_version ();
-int eaccess_stat ();
+int euidaccess ();
 int full_write ();
 
-static int do_copy ();
-static int copy ();
-static int copy_dir ();
-static int make_path_private ();
-static int copy_reg ();
-static int re_protect ();
+static int do_copy __P ((int argc, char **argv));
+static int copy __P ((const char *src_path, const char *dst_path, int new_dst,
+		      dev_t device, struct dir_list *ancestors));
+static int copy_dir __P ((const char *src_path_in, const char *dst_path_in,
+			  int new_dst, const struct stat *src_sb,
+			  struct dir_list *ancestors));
+static int make_path_private __P ((const char *const_dirpath, int src_offset,
+				   int mode, const char *verbose_fmt_string,
+				   struct dir_attr **attr_list, int *new_dst));
+static int copy_reg __P ((const char *src_path, const char *dst_path));
+static int re_protect __P ((const char *const_dst_path, int src_offset,
+			    struct dir_attr *attr_list));
 
 /* Initial number of entries in each hash table entry's table of inodes.  */
 #define INITIAL_HASH_MODULE 100
@@ -115,6 +149,19 @@ static int flag_update = 0;
 /* If nonzero, display the names of the files before copying them. */
 static int flag_verbose = 0;
 
+static char const *const sparse_type_string[] =
+{
+  "never", "auto", "always", 0
+};
+
+static enum Sparse_type const sparse_type[] =
+{
+  SPARSE_NEVER, SPARSE_AUTO, SPARSE_ALWAYS
+};
+
+/* Control creation of sparse files.  */
+static int flag_sparse = SPARSE_AUTO;
+
 /* The error code to return to the system. */
 static int exit_status = 0;
 
@@ -124,10 +171,10 @@ static int umask_kill;
 /* This process's effective user ID.  */
 static uid_t myeuid;
 
-/* If non-zero, display usage information and exit.  */
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output and exit.  */
+/* If nonzero, print the version on standard output and exit.  */
 static int show_version;
 
 static struct option const long_opts[] =
@@ -135,6 +182,7 @@ static struct option const long_opts[] =
   {"archive", no_argument, NULL, 'a'},
   {"backup", no_argument, NULL, 'b'},
   {"force", no_argument, NULL, 'f'},
+  {"sparse", required_argument, NULL, 2},
   {"interactive", no_argument, NULL, 'i'},
   {"link", no_argument, NULL, 'l'},
   {"no-dereference", no_argument, &flag_dereference, 0},
@@ -152,17 +200,19 @@ static struct option const long_opts[] =
   {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
-
-void
-main (argc, argv)
-     int argc;
-     char *argv[];
+
+int
+main (int argc, char **argv)
 {
   int c;
   int make_backups = 0;
   char *version;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   myeuid = geteuid ();
 
   version = getenv ("SIMPLE_BACKUP_SUFFIX");
@@ -185,6 +235,21 @@ main (argc, argv)
 	case 0:
 	  break;
 
+	case 2:
+	  {
+	    int i;
+
+	    /* --sparse={never,auto,always}  */
+	    i = argmatch (optarg, sparse_type_string);
+	    if (i < 0)
+	      {
+		invalid_arg (_("sparse type"), optarg, i);
+		usage (2, NULL);
+	      }
+	    flag_sparse = sparse_type[i];
+	  }
+	  break;
+
 	case 'a':		/* Like -dpR. */
 	  flag_dereference = 0;
 	  flag_preserve = 1;
@@ -236,7 +301,7 @@ main (argc, argv)
 #ifdef S_ISLNK
 	  flag_symbolic_link = 1;
 #else
-	  error (1, 0, "symbolic links are not supported on this system");
+	  error (1, 0, _("symbolic links are not supported on this system"));
 #endif
 	  break;
 
@@ -267,7 +332,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("%s\n", version_string);
+      printf ("cp - %s\n", PACKAGE_VERSION);
       exit (0);
     }
 
@@ -275,7 +340,7 @@ main (argc, argv)
     usage (0, NULL);
 
   if (flag_hard_link && flag_symbolic_link)
-    usage (2, "cannot make both hard and symbolic links");
+    usage (2, _("cannot make both hard and symbolic links"));
 
   if (make_backups)
     backup_type = get_version (version);
@@ -287,9 +352,9 @@ main (argc, argv)
      of `stat' to call.  */
 
   if (flag_dereference)
-    xstat = safe_stat;
+    xstat = stat;
   else
-    xstat = safe_lstat;
+    xstat = lstat;
 
   /* Allocate space for remembering copied and created files.  */
 
@@ -299,14 +364,34 @@ main (argc, argv)
 
   exit (exit_status);
 }
-
+
+/* Concatenate two pathname components, DIR and BASE, in newly-allocated
+   storage and return the result.  Be careful that in the result they are
+   separated by a slash.  That is, if DIR ends with a slash or if BASE
+   begins with one, don't add a separating slash.  Otherwise, add one.  */
+
+static char *
+path_concat (const char *dir, const char *base)
+{
+  char *dir_end;
+  char *p_concat;
+
+  assert (strlen (dir) > 0);
+  p_concat = xmalloc (strlen (dir) + strlen (base) + 2);
+  dir_end = stpcpy (p_concat, dir);
+  if (*(dir_end - 1) == '/')
+    --dir_end;
+  else if (*base == '/')
+    ++base;
+  stpcpy (stpcpy (dir_end, "/"), base);
+  return p_concat;
+}
+
 /* Scan the arguments, and copy each by calling copy.
    Return 0 if successful, 1 if any errors occur. */
 
 static int
-do_copy (argc, argv)
-     int argc;
-     char *argv[];
+do_copy (int argc, char **argv)
 {
   char *dest;
   struct stat sb;
@@ -314,13 +399,13 @@ do_copy (argc, argv)
   int ret = 0;
 
   if (optind >= argc)
-    usage (2, "missing file arguments");
+    usage (2, _("missing file arguments"));
   if (optind >= argc - 1)
-    usage (2, "missing file argument");
+    usage (2, _("missing destination file"));
 
   dest = argv[argc - 1];
 
-  if (SAFE_LSTAT (dest, &sb))
+  if (lstat (dest, &sb))
     {
       if (errno != ENOENT)
 	{
@@ -337,7 +422,7 @@ do_copy (argc, argv)
       /* If `dest' is not a symlink to a nonexistent file, use
 	 the results of stat instead of lstat, so we can copy files
 	 into symlinks to directories. */
-      if (SAFE_STAT (dest, &sbx) == 0)
+      if (stat (dest, &sbx) == 0)
 	sb = sbx;
     }
 
@@ -362,17 +447,16 @@ do_copy (argc, argv)
 	  if (flag_path)
 	    {
 	      /* Append all of `arg' to `dest'.  */
-	      dst_path = xmalloc (strlen (dest) + strlen (arg) + 2);
-	      stpcpy (stpcpy (stpcpy (dst_path, dest), "/"), arg);
+	      dst_path = path_concat (dest, arg);
 
 	      /* For --parents, we have to make sure that the directory
 	         dirname (dst_path) exists.  We may have to create a few
 	         leading directories. */
 	      parent_exists = !make_path_private (dst_path,
-					  strlen (dest) + 1, 0700,
-					  flag_verbose ? "%s -> %s\n" :
-					  (char *) NULL,
-					  &attr_list, &new_dst);
+						  strlen (dest) + 1, 0700,
+						  (flag_verbose
+						   ? "%s -> %s\n" : NULL),
+						  &attr_list, &new_dst);
 	    }
   	  else
   	    {
@@ -380,18 +464,14 @@ do_copy (argc, argv)
 
 	      ap = basename (arg);
 	      /* For `cp -R source/.. dest', don't copy into `dest/..'. */
-	      if (!strcmp (ap, ".."))
-		dst_path = xstrdup (dest);
-	      else
-		{
-		  dst_path = xmalloc (strlen (dest) + strlen (ap) + 2);
-		  stpcpy (stpcpy (stpcpy (dst_path, dest), "/"), ap);
-		}
+	      dst_path = (STREQ (ap, "..")
+			  ? xstrdup (dest)
+			  : path_concat (dest, ap));
 	    }
 
 	  if (!parent_exists)
 	    {
-	      /* make_path_private failed, so we shouldn't even attempt the copy. */
+	      /* make_path_private failed, so don't even attempt the copy. */
 	      ret = 1;
   	    }
 	  else
@@ -401,8 +481,7 @@ do_copy (argc, argv)
 
 	      if (flag_path)
 		{
-		  ret |= re_protect (dst_path, strlen (dest) + 1,
-					attr_list);
+		  ret |= re_protect (dst_path, strlen (dest) + 1, attr_list);
 		}
 	    }
 
@@ -420,16 +499,34 @@ do_copy (argc, argv)
       struct stat source_stats;
 
       if (flag_path)
-	usage (2, "when preserving paths, last argument must be a directory");
+	usage (2,
+	       _("when preserving paths, last argument must be a directory"));
 
       source = argv[optind];
 
+      /* When the force and backup options have been specified and
+	 the source and destination are the same name for an existing
+	 regular file, convert the user's command, e.g.,
+	 `cp --force --backup foo foo' to `cp --force foo fooSUFFIX'
+	 where SUFFIX is determined by any version control options used.  */
+
+      if (flag_force
+	  && backup_type != none
+	  && STREQ (source, dest)
+	  && !new_dst && S_ISREG (sb.st_mode))
+	 {
+	   backup_type = none;
+	   new_dest = find_backup_file_name (dest);
+	   if (new_dest == NULL)
+	     error (1, 0, _("virtual memory exhausted"));
+	 }
+
       /* When the destination is specified with a trailing slash and the
 	 source exists but is not a directory, convert the user's command
 	 `cp source dest/' to `cp source dest/basename(source)'.  */
 
-      if (dest[strlen (dest) - 1] == '/'
-	  && SAFE_LSTAT (source, &source_stats) == 0
+      else if (dest[strlen (dest) - 1] == '/'
+	  && lstat (source, &source_stats) == 0
 	  && !S_ISDIR (source_stats.st_mode))
 	{
 	  char *source_base;
@@ -453,11 +550,11 @@ do_copy (argc, argv)
     }
   else
     usage (2,
-	   "when copying multiple files, last argument must be a directory");
+	 _("when copying multiple files, last argument must be a directory"));
 }
 
 /* Copy the file SRC_PATH to the file DST_PATH.  The files may be of
-   any type.  NEW_DST should be non-zero if the file DST_PATH cannot
+   any type.  NEW_DST should be nonzero if the file DST_PATH cannot
    exist because its parent directory was just created; NEW_DST should
    be zero if DST_PATH might already exist.  DEVICE is the device
    number of the parent directory, or 0 if the parent of this file is
@@ -466,12 +563,8 @@ do_copy (argc, argv)
    Return 0 if successful, 1 if an error occurs. */
 
 static int
-copy (src_path, dst_path, new_dst, device, ancestors)
-     char *src_path;
-     char *dst_path;
-     int new_dst;
-     dev_t device;
-     struct dir_list *ancestors;
+copy (const char *src_path, const char *dst_path, int new_dst, dev_t device,
+      struct dir_list *ancestors)
 {
   struct stat src_sb;
   struct stat dst_sb;
@@ -507,7 +600,7 @@ copy (src_path, dst_path, new_dst, device, ancestors)
 
   if (S_ISDIR (src_type) && !flag_recursive)
     {
-      error (0, 0, "%s: omitting directory", src_path);
+      error (0, 0, _("%s: omitting directory"), src_path);
       return 1;
     }
 
@@ -532,7 +625,7 @@ copy (src_path, dst_path, new_dst, device, ancestors)
 	      if (flag_hard_link)
 		return 0;
 
-	      error (0, 0, "`%s' and `%s' are the same file",
+	      error (0, 0, _("`%s' and `%s' are the same file"),
 		     src_path, dst_path);
 	      return 1;
 	    }
@@ -542,7 +635,7 @@ copy (src_path, dst_path, new_dst, device, ancestors)
 	      if (S_ISDIR (dst_sb.st_mode))
 		{
 		  error (0, 0,
-			 "%s: cannot overwrite directory with non-directory",
+		       _("%s: cannot overwrite directory with non-directory"),
 			 dst_path);
 		  return 1;
 		}
@@ -555,13 +648,13 @@ copy (src_path, dst_path, new_dst, device, ancestors)
 	    {
 	      if (flag_interactive)
 		{
-		  if (eaccess_stat (&dst_sb, W_OK, dst_path) != 0)
+		  if (euidaccess (dst_path, W_OK) != 0)
 		    fprintf (stderr,
-			     "%s: overwrite `%s', overriding mode %04o? ",
+			     _("%s: overwrite `%s', overriding mode %04o? "),
 			     program_name, dst_path,
 			     (unsigned int) (dst_sb.st_mode & 07777));
 		  else
-		    fprintf (stderr, "%s: overwrite `%s'? ",
+		    fprintf (stderr, _("%s: overwrite `%s'? "),
 			     program_name, dst_path);
 		  if (!yesno ())
 		    return 0;
@@ -572,7 +665,20 @@ copy (src_path, dst_path, new_dst, device, ancestors)
 	    {
 	      char *tmp_backup = find_backup_file_name (dst_path);
 	      if (tmp_backup == NULL)
-		error (1, 0, "virtual memory exhausted");
+		error (1, 0, _("virtual memory exhausted"));
+
+	      /* Detect (and fail) when creating the backup file would
+		 destroy the source file.  Before, running the commands
+		 cd /tmp; rm -f a a~; : > a; echo A > a~; cp -b -V simple a~ a
+		 would leave two zero-length files: a and a~.  */
+	      if (STREQ (tmp_backup, src_path))
+		{
+		  error (0, 0,
+		   _("backing up `%s' would destroy source;  `%s' not copied"),
+			 dst_path, src_path);
+		  return 1;
+
+		}
 	      dst_backup = (char *) alloca (strlen (tmp_backup) + 1);
 	      strcpy (dst_backup, tmp_backup);
 	      free (tmp_backup);
@@ -580,7 +686,7 @@ copy (src_path, dst_path, new_dst, device, ancestors)
 		{
 		  if (errno != ENOENT)
 		    {
-		      error (0, errno, "cannot backup `%s'", dst_path);
+		      error (0, errno, _("cannot backup `%s'"), dst_path);
 		      return 1;
 		    }
 		  else
@@ -593,7 +699,7 @@ copy (src_path, dst_path, new_dst, device, ancestors)
 	      if (S_ISDIR (dst_sb.st_mode))
 		{
 		  /* Temporarily change mode to allow overwriting. */
-		  if (eaccess_stat (&dst_sb, W_OK | X_OK) != 0)
+		  if (euidaccess (dst_path, W_OK | X_OK) != 0)
 		    {
 		      if (chmod (dst_path, 0700))
 			{
@@ -608,7 +714,7 @@ copy (src_path, dst_path, new_dst, device, ancestors)
 		{
 		  if (unlink (dst_path) && errno != ENOENT)
 		    {
-		      error (0, errno, "cannot remove old link to `%s'",
+		      error (0, errno, _("cannot remove old link to `%s'"),
 			     dst_path);
 		      return 1;
 		    }
@@ -648,7 +754,7 @@ copy (src_path, dst_path, new_dst, device, ancestors)
 
       if (is_ancestor (&src_sb, ancestors))
 	{
-	  error (0, 0, "%s: cannot copy cyclic symbolic link", src_path);
+	  error (0, 0, _("%s: cannot copy cyclic symbolic link"), src_path);
 	  goto un_backup;
 	}
 
@@ -666,7 +772,7 @@ copy (src_path, dst_path, new_dst, device, ancestors)
 
 	  if (mkdir (dst_path, (src_mode & umask_kill) | 0700))
 	    {
-	      error (0, errno, "cannot create directory `%s'", dst_path);
+	      error (0, errno, _("cannot create directory `%s'"), dst_path);
 	      goto un_backup;
 	    }
 
@@ -690,8 +796,8 @@ copy (src_path, dst_path, new_dst, device, ancestors)
   else if (flag_symbolic_link)
     {
       if (*src_path == '/'
-	  || (!strncmp (dst_path, "./", 2) && index (dst_path + 2, '/') == 0)
-	  || index (dst_path, '/') == 0)
+	  || (!strncmp (dst_path, "./", 2) && strchr (dst_path + 2, '/') == 0)
+	  || strchr (dst_path, '/') == 0)
 	{
 	  if (symlink (src_path, dst_path))
 	    {
@@ -703,7 +809,8 @@ copy (src_path, dst_path, new_dst, device, ancestors)
       else
 	{
 	  error (0, 0,
-		 "%s: can only make relative symbolic links in current directory", dst_path);
+	   _("%s: can make relative symbolic links only in current directory"),
+		 dst_path);
 	  goto un_backup;
 	}
     }
@@ -712,7 +819,7 @@ copy (src_path, dst_path, new_dst, device, ancestors)
     {
       if (link (src_path, dst_path))
 	{
-	  error (0, errno, "cannot create link `%s'", dst_path);
+	  error (0, errno, _("cannot create link `%s'"), dst_path);
 	  goto un_backup;
 	}
       return 0;
@@ -733,7 +840,7 @@ copy (src_path, dst_path, new_dst, device, ancestors)
     {
       if (mkfifo (dst_path, src_mode & umask_kill))
 	{
-	  error (0, errno, "cannot create fifo `%s'", dst_path);
+	  error (0, errno, _("cannot create fifo `%s'"), dst_path);
 	  goto un_backup;
 	}
     }
@@ -747,7 +854,7 @@ copy (src_path, dst_path, new_dst, device, ancestors)
     {
       if (mknod (dst_path, src_mode & umask_kill, src_sb.st_rdev))
 	{
-	  error (0, errno, "cannot create special file `%s'", dst_path);
+	  error (0, errno, _("cannot create special file `%s'"), dst_path);
 	  goto un_backup;
 	}
     }
@@ -762,14 +869,14 @@ copy (src_path, dst_path, new_dst, device, ancestors)
       link_size = readlink (src_path, link_val, PATH_MAX + 1);
       if (link_size < 0)
 	{
-	  error (0, errno, "cannot read symbolic link `%s'", src_path);
+	  error (0, errno, _("cannot read symbolic link `%s'"), src_path);
 	  goto un_backup;
 	}
       link_val[link_size] = '\0';
 
       if (symlink (link_val, dst_path))
 	{
-	  error (0, errno, "cannot create symbolic link `%s'", dst_path);
+	  error (0, errno, _("cannot create symbolic link `%s'"), dst_path);
 	  goto un_backup;
 	}
       return 0;
@@ -777,7 +884,7 @@ copy (src_path, dst_path, new_dst, device, ancestors)
   else
 #endif
     {
-      error (0, 0, "%s: unknown file type", src_path);
+      error (0, 0, _("%s: unknown file type"), src_path);
       goto un_backup;
     }
 
@@ -835,7 +942,7 @@ un_backup:
   if (dst_backup)
     {
       if (rename (dst_backup, dst_path))
-	error (0, errno, "cannot un-backup `%s'", dst_path);
+	error (0, errno, _("cannot un-backup `%s'"), dst_path);
     }
   return 1;
 }
@@ -859,14 +966,9 @@ un_backup:
    permissions when done, otherwise 1. */
 
 static int
-make_path_private (const_dirpath, src_offset, mode, verbose_fmt_string,
-		   attr_list, new_dst)
-     char *const_dirpath;
-     int src_offset;
-     int mode;
-     char *verbose_fmt_string;
-     struct dir_attr **attr_list;
-     int *new_dst;
+make_path_private (const char *const_dirpath, int src_offset, int mode,
+		   const char *verbose_fmt_string, struct dir_attr **attr_list,
+		   int *new_dst)
 {
   struct stat stats;
   char *dirpath;		/* A copy of CONST_DIRPATH we can change. */
@@ -895,7 +997,7 @@ make_path_private (const_dirpath, src_offset, mode, verbose_fmt_string,
       slash = src;
       while (*slash == '/')
 	slash++;
-      while ((slash = index (slash, '/')))
+      while ((slash = strchr (slash, '/')))
 	{
 	  /* Add this directory to the list of directories whose modes need
 	     fixing later. */
@@ -917,7 +1019,7 @@ make_path_private (const_dirpath, src_offset, mode, verbose_fmt_string,
 	      new->is_new_dir = 1;
 	      if (mkdir (dirpath, mode))
 		{
-		  error (0, errno, "cannot make directory `%s'", dirpath);
+		  error (0, errno, _("cannot make directory `%s'"), dirpath);
 		  return 1;
 		}
 	      else
@@ -928,7 +1030,7 @@ make_path_private (const_dirpath, src_offset, mode, verbose_fmt_string,
 	    }
 	  else if (!S_ISDIR (stats.st_mode))
 	    {
-	      error (0, 0, "`%s' exists but is not a directory", dirpath);
+	      error (0, 0, _("`%s' exists but is not a directory"), dirpath);
 	      return 1;
 	    }
 	  else
@@ -949,7 +1051,7 @@ make_path_private (const_dirpath, src_offset, mode, verbose_fmt_string,
 
   else if (!S_ISDIR (stats.st_mode))
     {
-      error (0, 0, "`%s' exists but is not a directory", dst_dirname);
+      error (0, 0, _("`%s' exists but is not a directory"), dst_dirname);
       return 1;
     }
   else
@@ -980,10 +1082,8 @@ make_path_private (const_dirpath, src_offset, mode, verbose_fmt_string,
    when done, otherwise 1. */
 
 static int
-re_protect (const_dst_path, src_offset, attr_list)
-     char *const_dst_path;
-     int src_offset;
-     struct dir_attr *attr_list;
+re_protect (const char *const_dst_path, int src_offset,
+	    struct dir_attr *attr_list)
 {
   struct dir_attr *p;
   char *dst_path;		/* A copy of CONST_DST_PATH we can change. */
@@ -1047,18 +1147,14 @@ re_protect (const_dst_path, src_offset, attr_list)
 }
 
 /* Read the contents of the directory SRC_PATH_IN, and recursively
-   copy the contents to DST_PATH_IN.  NEW_DST is non-zero if
+   copy the contents to DST_PATH_IN.  NEW_DST is nonzero if
    DST_PATH_IN is a directory that was created previously in the
    recursion.   SRC_SB and ANCESTORS describe SRC_PATH_IN.
    Return 0 if successful, -1 if an error occurs. */
 
 static int
-copy_dir (src_path_in, dst_path_in, new_dst, src_sb, ancestors)
-     char *src_path_in;
-     char *dst_path_in;
-     int new_dst;
-     struct stat *src_sb;
-     struct dir_list *ancestors;
+copy_dir (const char *src_path_in, const char *dst_path_in, int new_dst,
+	  const struct stat *src_sb, struct dir_list *ancestors)
 {
   char *name_space;
   char *namep;
@@ -1076,7 +1172,7 @@ copy_dir (src_path_in, dst_path_in, new_dst, src_sb, ancestors)
 	  return -1;
 	}
       else
-	error (1, 0, "virtual memory exhausted");
+	error (1, 0, _("virtual memory exhausted"));
     }
 
   namep = name_space;
@@ -1111,9 +1207,7 @@ copy_dir (src_path_in, dst_path_in, new_dst, src_sb, ancestors)
    Return 0 if successful, -1 if an error occurred. */
 
 static int
-copy_reg (src_path, dst_path)
-     char *src_path;
-     char *dst_path;
+copy_reg (const char *src_path, const char *dst_path)
 {
   char *buf;
   int buf_size;
@@ -1126,7 +1220,7 @@ copy_reg (src_path, dst_path)
   int return_val = 0;
   long n_read_total = 0;
   int last_write_made_hole = 0;
-  int make_holes = 0;
+  int make_holes = (flag_sparse == SPARSE_ALWAYS);
 
   source_desc = open (src_path, O_RDONLY);
   if (source_desc < 0)
@@ -1141,7 +1235,7 @@ copy_reg (src_path, dst_path)
   dest_desc = open (dst_path, O_WRONLY | O_CREAT | O_TRUNC, 0600);
   if (dest_desc < 0)
     {
-      error (0, errno, "cannot create regular file `%s'", dst_path);
+      error (0, errno, _("cannot create regular file `%s'"), dst_path);
       return_val = -1;
       goto ret2;
     }
@@ -1158,9 +1252,10 @@ copy_reg (src_path, dst_path)
   buf_size = ST_BLKSIZE (sb);
 
 #ifdef HAVE_ST_BLOCKS
-  if (S_ISREG (sb.st_mode))
+  if (flag_sparse == SPARSE_AUTO && S_ISREG (sb.st_mode))
     {
-      /* Find out whether the file contains any sparse blocks. */
+      /* Use a heuristic to determine whether SRC_PATH contains any
+	 sparse blocks. */
 
       if (fstat (source_desc, &sb))
 	{
@@ -1172,7 +1267,8 @@ copy_reg (src_path, dst_path)
       /* If the file has fewer blocks than would normally
 	 be needed for a file of its size, then
 	 at least one of the blocks in the file is a hole. */
-      if (S_ISREG (sb.st_mode) && sb.st_size > sb.st_blocks * DEV_BSIZE)
+      if (S_ISREG (sb.st_mode)
+	  && (size_t) (sb.st_size / 512) > (size_t) ST_NBLOCKS (sb))
 	make_holes = 1;
     }
 #endif
@@ -1204,13 +1300,13 @@ copy_reg (src_path, dst_path)
 	{
 	  buf[n_read] = 1;	/* Sentinel to stop loop.  */
 
-	  /* Find first non-zero *word*, or the word with the sentinel.  */
+	  /* Find first nonzero *word*, or the word with the sentinel.  */
 
 	  ip = (int *) buf;
 	  while (*ip++ == 0)
 	    ;
 
-	  /* Find the first non-zero *byte*, or the sentinel.  */
+	  /* Find the first nonzero *byte*, or the sentinel.  */
 
 	  cp = (char *) (ip - 1);
 	  while (*cp++ == 0)
