@@ -1,5 +1,5 @@
 /* dd -- convert a file while copying it.
-   Copyright (C) 1985, 1990, 1991 Free Software Foundation, Inc.
+   Copyright (C) 85, 90, 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Written by Paul Rubin, David MacKenzie, and Stuart Kemp. */
 
@@ -44,8 +44,8 @@
 				cbs, replacing newline with trailing spaces.
    unblock			Replace trailing spaces in cbs-sized block
 				with newline.
-   lcase			Change uppercase characters to lowercase.
-   ucase			Change lowercase characters to uppercase.
+   lcase			Change upper case characters to lower case.
+   ucase			Change lower case characters to upper case.
    swab				Swap every pair of input bytes.
 				Unlike the Unix dd, this works when an odd
 				number of bytes are read.
@@ -53,35 +53,17 @@
    sync				Pad every input block to size of ibs with
 				trailing NULs. */
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
-
 #include <stdio.h>
-#include <ctype.h>
 
-#if !defined (isascii) || defined (STDC_HEADERS)
-#undef isascii
-#define isascii(c) 1
-#endif
-
-#define ISLOWER(c) (isascii (c) && islower (c))
-#define ISUPPER(c) (isascii (c) && isupper (c))
-#define ISDIGIT(c) (isascii (c) && isdigit (c))
+#define SWAB_ALIGN_OFFSET 2
 
 #include <sys/types.h>
 #include <signal.h>
 #include <getopt.h>
 
 #include "system.h"
-#include "version.h"
+#include "error.h"
 
 #define equal(p, q) (strcmp ((p),(q)) == 0)
 #define max(a, b) ((a) > (b) ? (a) : (b))
@@ -109,24 +91,26 @@
 #define C_TWOBUFS 04000
 
 char *xmalloc ();
-void error ();
+int safe_read ();
+int full_write ();
 
-static RETSIGTYPE interrupt_handler ();
-static int bit_count ();
-static int parse_integer ();
-static void apply_translations ();
-static void copy ();
-static void copy_simple ();
-static void copy_with_block ();
-static void copy_with_unblock ();
-static void parse_conversion ();
-static void print_stats ();
-static void translate_charset ();
-static void quit ();
-static void scanargs ();
-static void skip ();
-static void usage ();
-static void write_output ();
+static RETSIGTYPE interrupt_handler __P ((int));
+static int bit_count __P ((register unsigned int i));
+static int parse_integer __P ((char *str));
+static void apply_translations __P ((void));
+static void copy __P ((void));
+static void copy_simple __P ((unsigned char *buf, int nread));
+static void copy_with_block __P ((unsigned char *buf, int nread));
+static void copy_with_unblock __P ((unsigned char *buf, int nread));
+static void parse_conversion __P ((char *str));
+static void print_stats __P ((void));
+static void translate_charset __P ((const unsigned char *new_trans));
+static void quit __P ((int code));
+static void scanargs __P ((int argc, char **argv));
+static void skip __P ((int fdesc, char *file, long int records,
+		       long int blocksize, unsigned char *buf));
+static void usage __P ((int status));
+static void write_output __P ((void));
 
 /* The name this program was run with. */
 char *program_name;
@@ -320,10 +304,10 @@ static unsigned char const ebcdic_to_ascii[] =
   070, 071, 0372, 0373, 0374, 0375, 0376, 0377
 };
 
-/* If non-zero, display usage information and exit.  */
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output and exit.  */
+/* If nonzero, print the version on standard output and exit.  */
 static int show_version;
 
 static struct option const long_options[] =
@@ -333,10 +317,8 @@ static struct option const long_options[] =
   {0, 0, 0, 0}
 };
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+int
+main (int argc, char **argv)
 {
 #ifdef _POSIX_VERSION
   struct sigaction sigact;
@@ -344,6 +326,9 @@ main (argc, argv)
   int i;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
   /* Initialize translation table to identity translation. */
   for (i = 0; i < 256; i++)
@@ -354,7 +339,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("%s\n", version_string);
+      printf ("dd - %s\n", PACKAGE_VERSION);
       exit (0);
     }
 
@@ -370,10 +355,12 @@ main (argc, argv)
 	error (1, errno, "%s", input_file);
     }
   else
-    input_file = "standard input";
+    input_file = _("standard input");
 
   if (input_fd == output_fd)
-    error (1, 0, "standard %s is closed", input_fd == 0 ? "input" : "output");
+    error (1, 0, _("%s is closed"), (input_fd == 0
+				     ? _("standard input")
+				     : _("standard output")));
 
   if (output_file != NULL)
     {
@@ -393,7 +380,7 @@ main (argc, argv)
 #endif
     }
   else
-    output_file = "standard output";
+    output_file = _("standard output");
 
 #ifdef _POSIX_VERSION
   sigaction (SIGINT, NULL, &sigact);
@@ -404,9 +391,19 @@ main (argc, argv)
       sigact.sa_flags = 0;
       sigaction (SIGINT, &sigact, NULL);
     }
+  sigaction (SIGPIPE, NULL, &sigact);
+  if (sigact.sa_handler != SIG_IGN)
+    {
+      sigact.sa_handler = interrupt_handler;
+      sigemptyset (&sigact.sa_mask);
+      sigact.sa_flags = 0;
+      sigaction (SIGPIPE, &sigact, NULL);
+    }
 #else				/* !_POSIX_VERSION */
   if (signal (SIGINT, SIG_IGN) != SIG_IGN)
     signal (SIGINT, interrupt_handler);
+  if (signal (SIGPIPE, SIG_IGN) != SIG_IGN)
+    signal (SIGPIPE, interrupt_handler);
 #endif				/* !_POSIX_VERSION */
   copy ();
 }
@@ -416,24 +413,24 @@ main (argc, argv)
    bytes of the data at a time in BUF, if necessary. */
 
 static void
-skip (fdesc, file, records, blocksize, buf)
-     int fdesc;
-     char *file;
-     long records;
-     long blocksize;
-     char *buf;
+skip (int fdesc, char *file, long int records, long int blocksize,
+      unsigned char *buf)
 {
   struct stat stats;
 
   /* Use fstat instead of checking for errno == ESPIPE because
      lseek doesn't work on some special files but doesn't return an
      error, either. */
+  /* FIXME: can this really happen?  What system?  */
   if (fstat (fdesc, &stats))
     {
       error (0, errno, "%s", file);
       quit (1);
     }
 
+  /* FIXME: why use lseek only on regular files?
+     Better: try lseek and if an error indicates it was an inappropriate
+     operation, fall back on using read.  */
   if (S_ISREG (stats.st_mode))
     {
       if (lseek (fdesc, records * blocksize, SEEK_SET) < 0)
@@ -446,13 +443,19 @@ skip (fdesc, file, records, blocksize, buf)
     {
       while (records-- > 0)
 	{
-	  if (read (fdesc, buf, blocksize) < 0)
+	  int nread;
+
+	  nread = safe_read (fdesc, buf, blocksize);
+	  if (nread < 0)
 	    {
 	      error (0, errno, "%s", file);
 	      quit (1);
 	    }
-	  /* FIXME If fewer bytes were read than requested, meaning that
-	     EOF was reached, POSIX wants the output file padded with NULs. */
+	  /* POSIX doesn't say what to do when dd detects it has been
+	     asked to skip past EOF, so I assume it's non-fatal.
+	     FIXME: maybe give a warning.  */
+	  if (nread == 0)
+	    break;
 	}
     }
 }
@@ -461,9 +464,7 @@ skip (fdesc, file, records, blocksize, buf)
    to the NREAD bytes in BUF.  */
 
 static void
-translate_buffer (buf, nread)
-     unsigned char *buf;
-     int nread;
+translate_buffer (unsigned char *buf, int nread)
 {
   register unsigned char *cp;
   register int i;
@@ -484,9 +485,7 @@ static unsigned char saved_char;
    next call.   Return the new start of the BUF buffer.  */
 
 static unsigned char *
-swab_buffer (buf, nread)
-     unsigned char *buf;
-     int *nread;
+swab_buffer (unsigned char *buf, int *nread)
 {
   unsigned char *bufstart = buf;
   register unsigned char *cp;
@@ -530,14 +529,20 @@ static int col = 0;
 /* The main loop.  */
 
 static void
-copy ()
+copy (void)
 {
   unsigned char *ibuf, *bufstart; /* Input buffer. */
   int nread;			/* Bytes read in the current block. */
   int exit_status = 0;
 
-  /* Leave an extra byte at the beginning and end of `ibuf' for conv=swab.  */
-  ibuf = (unsigned char *) xmalloc (input_blocksize + 2) + 1;
+  /* Leave at least one extra byte at the beginning and end of `ibuf'
+     for conv=swab, but keep the buffer address even.  But some peculiar
+     device drivers work only with word-aligned buffers, so leave an
+     extra two bytes.  */
+
+  ibuf = (unsigned char *) xmalloc (input_blocksize + 2 * SWAB_ALIGN_OFFSET);
+  ibuf += SWAB_ALIGN_OFFSET;
+
   if (conversions_mask & C_TWOBUFS)
     obuf = (unsigned char *) xmalloc (output_blocksize);
   else
@@ -561,9 +566,9 @@ copy ()
 	 whatever data we are able to read is followed by zeros.
 	 This minimizes data loss. */
       if ((conversions_mask & C_SYNC) && (conversions_mask & C_NOERROR))
-	bzero (ibuf, input_blocksize);
+	memset ((char *) ibuf, 0, input_blocksize);
 
-      nread = read (input_fd, ibuf, input_blocksize);
+      nread = safe_read (input_fd, ibuf, input_blocksize);
 
       if (nread == 0)
 	break;			/* EOF.  */
@@ -598,7 +603,7 @@ copy ()
 	    {
 	      if (!(conversions_mask & C_NOERROR))
 		/* If C_NOERROR, we zeroed the block before reading. */
-		bzero (ibuf + nread, input_blocksize - nread);
+		memset ((char *) (ibuf + nread), 0, input_blocksize - nread);
 	      nread = input_blocksize;
 	    }
 	}
@@ -607,12 +612,10 @@ copy ()
 
       if (ibuf == obuf)		/* If not C_TWOBUFS. */
 	{
-	  int nwritten = write (output_fd, obuf, nread);
-	  if (nwritten != nread)
+	  int nwritten = full_write (output_fd, obuf, nread);
+	  if (nwritten < 0)
 	    {
 	      error (0, errno, "%s", output_file);
-	      if (nwritten > 0)
-		w_partial++;
 	      quit (1);
 	    }
 	  else if (nread == input_blocksize)
@@ -671,17 +674,17 @@ copy ()
   /* Write out the last block. */
   if (oc > 0)
     {
-      int nwritten = write (output_fd, obuf, oc);
+      int nwritten = full_write (output_fd, obuf, oc);
       if (nwritten > 0)
 	w_partial++;
-      if (nwritten != oc)
+      if (nwritten < 0)
 	{
 	  error (0, errno, "%s", output_file);
 	  quit (1);
 	}
     }
 
-  free (ibuf - 1);
+  free (ibuf - SWAB_ALIGN_OFFSET);
   if (obuf != ibuf)
     free (obuf);
 
@@ -691,9 +694,7 @@ copy ()
 /* Copy NREAD bytes of BUF, with no conversions.  */
 
 static void
-copy_simple (buf, nread)
-     unsigned char *buf;
-     int nread;
+copy_simple (unsigned char *buf, int nread)
 {
   int nfree;			/* Number of unused bytes in `obuf'.  */
   unsigned char *start = buf; /* First uncopied char in BUF.  */
@@ -704,7 +705,7 @@ copy_simple (buf, nread)
       if (nfree > nread)
 	nfree = nread;
 
-      bcopy (start, obuf + oc, nfree);
+      memcpy ((char *) (obuf + oc), (char *) start, nfree);
 
       nread -= nfree;		/* Update the number of bytes left to copy. */
       start += nfree;
@@ -720,9 +721,7 @@ copy_simple (buf, nread)
    replacing the newline with trailing spaces).  */
 
 static void
-copy_with_block (buf, nread)
-     unsigned char *buf;
-     int nread;
+copy_with_block (unsigned char *buf, int nread)
 {
   register int i;
 
@@ -754,9 +753,7 @@ copy_with_block (buf, nread)
    with a newline).  */
 
 static void
-copy_with_unblock (buf, nread)
-     unsigned char *buf;
-     int nread;
+copy_with_unblock (unsigned char *buf, int nread)
 {
   register int i;
   register unsigned char c;
@@ -791,9 +788,9 @@ copy_with_unblock (buf, nread)
 /* Write, then empty, the output buffer `obuf'. */
 
 static void
-write_output ()
+write_output (void)
 {
-  int nwritten = write (output_fd, obuf, output_blocksize);
+  int nwritten = full_write (output_fd, obuf, output_blocksize);
   if (nwritten != output_blocksize)
     {
       error (0, errno, "%s", output_file);
@@ -807,9 +804,7 @@ write_output ()
 }
 
 static void
-scanargs (argc, argv)
-     int argc;
-     char **argv;
+scanargs (int argc, char **argv)
 {
   int i, n;
   int c;
@@ -831,10 +826,10 @@ scanargs (argc, argv)
       char *name, *val;
 
       name = argv[i];
-      val = index (name, '=');
+      val = strchr (name, '=');
       if (val == NULL)
 	{
-	  error (0, 0, "unrecognized option `%s'", name);
+	  error (0, 0, _("unrecognized option `%s'"), name);
 	  usage (1);
 	}
       *val++ = '\0';
@@ -849,7 +844,7 @@ scanargs (argc, argv)
 	{
 	  n = parse_integer (val);
 	  if (n < 0)
-	    error (1, 0, "invalid number `%s'", val);
+	    error (1, 0, _("invalid number `%s'"), val);
 
 	  if (equal (name, "ibs"))
 	    {
@@ -873,7 +868,7 @@ scanargs (argc, argv)
 	    max_records = n;
 	  else
 	    {
-	      error (0, 0, "unrecognized option `%s=%s'", name, val);
+	      error (0, 0, _("unrecognized option `%s=%s'"), name, val);
 	      usage (1);
 	    }
 	}
@@ -896,9 +891,10 @@ scanargs (argc, argv)
    optionally multiplied by various values.
    Return -1 if STR does not represent a number in this format. */
 
+/* FIXME: use xstrtou?l */
+
 static int
-parse_integer (str)
-     char *str;
+parse_integer (char *str)
 {
   register int n = 0;
   register int temp;
@@ -940,15 +936,14 @@ loop:
 /* Interpret one "conv=..." option. */
 
 static void
-parse_conversion (str)
-     char *str;
+parse_conversion (char *str)
 {
   char *new;
   int i;
 
   do
     {
-      new = index (str, ',');
+      new = strchr (str, ',');
       if (new != NULL)
 	*new++ = '\0';
       for (i = 0; conversions[i].convname != NULL; i++)
@@ -959,7 +954,7 @@ parse_conversion (str)
 	  }
       if (conversions[i].convname == NULL)
 	{
-	  error (0, 0, "%s: invalid conversion", str);
+	  error (0, 0, _("%s: invalid conversion"), str);
 	  usage (1);
 	}
       str = new;
@@ -969,7 +964,7 @@ parse_conversion (str)
 /* Fix up translation table. */
 
 static void
-apply_translations ()
+apply_translations (void)
 {
   int i;
 
@@ -979,8 +974,8 @@ apply_translations ()
       || (MX (C_LCASE | C_UCASE) > 1)
       || (MX (C_UNBLOCK | C_SYNC) > 1))
     {
-      error (1, 0, "\
-only one conv in {ascii,ebcdic,ibm}, {lcase,ucase}, {block,unblock}, {unblock,sync}");
+      error (1, 0, _("\
+only one conv in {ascii,ebcdic,ibm}, {lcase,ucase}, {block,unblock}, {unblock,sync}"));
     }
 #undef MX
 
@@ -1017,8 +1012,7 @@ only one conv in {ascii,ebcdic,ibm}, {lcase,ucase}, {block,unblock}, {unblock,sy
 }
 
 static void
-translate_charset (new_trans)
-     unsigned char *new_trans;
+translate_charset (const unsigned char *new_trans)
 {
   int i;
 
@@ -1030,8 +1024,7 @@ translate_charset (new_trans)
 /* Return the number of 1 bits in `i'. */
 
 static int
-bit_count (i)
-     register unsigned int i;
+bit_count (register unsigned int i)
 {
   register int set_bits;
 
@@ -1041,45 +1034,62 @@ bit_count (i)
 }
 
 static void
-print_stats ()
+print_stats (void)
 {
-  fprintf (stderr, "%u+%u records in\n", r_full, r_partial);
-  fprintf (stderr, "%u+%u records out\n", w_full, w_partial);
+  fprintf (stderr, _("%u+%u records in\n"), r_full, r_partial);
+  fprintf (stderr, _("%u+%u records out\n"), w_full, w_partial);
   if (r_truncate > 0)
-    fprintf (stderr, "%u truncated record%s\n", r_truncate,
-	     r_truncate == 1 ? "" : "s");
+    fprintf (stderr, "%u %s\n", r_truncate,
+	     (r_truncate == 1
+	      ? _("truncated record")
+	      : _("truncated records")));
 }
 
 static void
-quit (code)
-     int code;
+cleanup (void)
 {
-  int errcode = code ? code : 1;
   print_stats ();
   if (close (input_fd) < 0)
-    error (errcode, errno, "%s", input_file);
+    error (1, errno, "%s", input_file);
   if (close (output_fd) < 0)
-    error (errcode, errno, "%s", output_file);
+    error (1, errno, "%s", output_file);
+}
+
+static void
+quit (int code)
+{
+  cleanup ();
   exit (code);
 }
 
 static RETSIGTYPE
-interrupt_handler ()
+interrupt_handler (int sig)
 {
-  quit (1);
+#ifdef SA_INTERRUPT
+  struct sigaction sigact;
+
+  sigact.sa_handler = SIG_DFL;
+  sigemptyset (&sigact.sa_mask);
+  sigact.sa_flags = 0;
+  sigaction (sig, &sigact, NULL);
+#else				/* !SA_INTERRUPT */
+  signal (sig, SIG_DFL);
+#endif				/* SA_INTERRUPT */
+  cleanup ();
+  kill (getpid (), sig);
 }
 
 static void
-usage (status)
-     int status;
+usage (int status)
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("Usage: %s [OPTION]...\n", program_name);
-      printf ("\
+      printf (_("Usage: %s [OPTION]...\n"), program_name);
+      printf (_("\
+Copy a file, converting and formatting according to the options.\n\
 \n\
   bs=BYTES        force ibs=BYTES and obs=BYTES\n\
   cbs=BYTES       convert BYTES bytes at a time\n\
@@ -1100,13 +1110,13 @@ by w for x2, by b for x512, by k for x1024.  Each KEYWORD may be:\n\
   ascii     from EBCDIC to ASCII\n\
   ebcdic    from ASCII to EBCDIC\n\
   ibm       from ASCII to alternated EBCDIC\n\
-  block     pad newline-terminated records with spaces to cbs-size \n\
+  block     pad newline-terminated records with spaces to cbs-size\n\
   unblock   replace trailing spaces in cbs-size records with newline\n\
-  lcase     change uppercase to lowercase\n\
-  ucase     change lowercase to uppercase\n\
+  lcase     change upper case to lower case\n\
+  ucase     change lower case to upper case\n\
   swab      swap every pair of input bytes\n\
   noerror   continue after read errors\n\
-  sync      pad every input block with NULs to ibs-size\n");
+  sync      pad every input block with NULs to ibs-size\n"));
     }
   exit (status);
 }
