@@ -1,5 +1,5 @@
 /* du -- summarize disk usage
-   Copyright (C) 1988, 1989, 1990, 1991 Free Software Foundation, Inc.
+   Copyright (C) 88, 89, 90, 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Differences from the Unix du:
    * Doesn't simply ignore the names of regular files given as arguments
@@ -25,8 +25,10 @@
    -c		Write a grand total of all of the arguments after all
 		arguments have been processed.  This can be used to find
 		out the disk usage of a directory, with some files excluded.
+   -h		Print sizes in human readable format (1k 234M 2G, etc).
    -k		Print sizes in kilobytes instead of 512 byte blocks
 		(the default required by POSIX).
+   -m		Print sizes in megabytes instead of 512 byte blocks
    -b		Print sizes in bytes.
    -S		Count the size of each directory separately, not including
 		the sizes of subdirectories.
@@ -34,7 +36,9 @@
    -L		Dereference all symbolic links.
 
    By tege@sics.se, Torbjorn Granlund,
-   and djm@ai.mit.edu, David MacKenzie.  */
+   and djm@ai.mit.edu, David MacKenzie.
+   Variable blocks added by lm@sgi.com.
+*/
 
 #ifdef _AIX
  #pragma alloca
@@ -44,10 +48,15 @@
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
+#include <assert.h>
+
 #include "system.h"
-#include "version.h"
-#include "safe-stat.h"
-#include "safe-lstat.h"
+#include "save-cwd.h"
+#include "error.h"
+
+#undef	convert_blocks
+#define	convert_blocks(b, size) (size == size_kilobytes ? ((b) + 1) / 2 : \
+    size == size_megabytes ? ((b) + 1024) / 2048 : (b))
 
 /* Initial number of entries in each hash table entry's table of inodes.  */
 #define INITIAL_HASH_MODULE 100
@@ -58,6 +67,11 @@
 /* Initial size to allocate for `path'.  */
 #define INITIAL_PATH_SIZE 100
 
+/* The maximum length of a human-readable string.  Be pessimistic
+   and assume `int' is 64-bits wide.  Converting 2^63 - 1 gives the
+   11-character string, 8589934592G.  */
+#define LONGEST_HUMAN_READABLE 11
+
 /* Hash structure for inode and device numbers.  The separate entry
    structure makes it easier to rehash "in place".  */
 
@@ -89,24 +103,24 @@ typedef struct
   char *text;			/* Pointer to the text.  */
 } *string, stringstruct;
 
+int stat ();
+int lstat ();
+
 char *savedir ();
-#ifndef HAVE_FCHDIR
-char *xgetcwd ();
-#endif
 char *xmalloc ();
 char *xrealloc ();
-void error ();
 
-static int hash_insert ();
-static int hash_insert2 ();
-static long count_entry ();
-static void du_files ();
-static void hash_init ();
-static void hash_reset ();
-static void str_concatc ();
-static void str_copyc ();
-static void str_init ();
-static void str_trunc ();
+static int hash_insert __P ((ino_t ino, dev_t dev));
+static int hash_insert2 __P ((struct htab *htab, ino_t ino, dev_t dev));
+static long count_entry __P ((char *ent, int top, dev_t last_dev));
+static void du_files __P ((char **files));
+static void hash_init __P ((unsigned int modulus,
+			    unsigned int entry_tab_size));
+static void hash_reset __P ((void));
+static void str_concatc __P ((string s1, char *cstr));
+static void str_copyc __P ((string s1, char *cstr));
+static void str_init __P ((string *s1, unsigned int size));
+static void str_trunc __P ((string s1, unsigned int length));
 
 /* Name under which this program was invoked.  */
 char *program_name;
@@ -136,9 +150,13 @@ enum output_size
 {
   size_blocks,			/* 512-byte blocks. */
   size_kilobytes,		/* 1K blocks. */
+  size_megabytes,		/* 1024K blocks. */
   size_bytes			/* 1-byte blocks. */
 };
 
+/* human style output */
+static int opt_human_readable;
+
 /* The units to count in. */
 static enum output_size output_size;
 
@@ -158,10 +176,10 @@ static int (*xstat) ();
 /* The exit status to use if we don't get any fatal errors. */
 static int exit_status;
 
-/* If non-zero, display usage information and exit.  */
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output and exit.  */
+/* If nonzero, print the version on standard output and exit.  */
 static int show_version;
 
 /* Grand total size of all args. */
@@ -174,7 +192,9 @@ static struct option const long_options[] =
   {"count-links", no_argument, &opt_count_all, 1},
   {"dereference", no_argument, NULL, 'L'},
   {"dereference-args", no_argument, &opt_dereference_arguments, 1},
+  {"human-readable", no_argument, NULL, 'h'},
   {"kilobytes", no_argument, NULL, 'k'},
+  {"megabytes", no_argument, NULL, 'm'},
   {"one-file-system", no_argument, &opt_one_file_system, 1},
   {"separate-dirs", no_argument, &opt_separate_dirs, 1},
   {"summarize", no_argument, &opt_summarize_only, 1},
@@ -185,54 +205,69 @@ static struct option const long_options[] =
 };
 
 static void
-usage (status, reason)
-     int status;
-     char *reason;
+usage (int status, char *reason)
 {
   if (reason != NULL)
     fprintf (status == 0 ? stdout : stderr, "%s: %s\n",
 	     program_name, reason);
 
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("Usage: %s [OPTION]... [PATH]...\n", program_name);
-      printf ("\
+      printf (_("Usage: %s [OPTION]... [FILE]...\n"), program_name);
+      printf (_("\
+Summarize disk usage of each FILE, recursively for directories.\n\
 \n\
-  -a, --all                write counts for all files, not just directories\n\
-  -b, --bytes              print size in bytes\n\
-  -c, --total              produce a grand total\n\
-  -k, --kilobytes          use 1024 blocks, not 512 despite POSIXLY_CORRECT\n\
-  -l, --count-links        count sizes many times if hard linked\n\
-  -s, --summarize          display only a total for each argument\n\
-  -x, --one-file-system    skip directories on different filesystems\n\
-  -D, --dereference-args   dereference PATHs when symbolic link\n\
-  -L, --dereference        dereference all symbolic links\n\
-  -S, --separate-dirs      do not include size of subdirectories\n\
-      --help               display this help and exit\n\
-      --version            output version information and exit\n");
+  -a, --all             write counts for all files, not just directories\n\
+  -b, --bytes           print size in bytes\n\
+  -c, --total           produce a grand total\n\
+  -h, --human-readable  print sizes in human readable format (e.g. 1K 234M 2G)\n\
+  -k, --kilobytes       use 1024-byte blocks, not 512 despite POSIXLY_CORRECT\n\
+  -l, --count-links     count sizes many times if hard linked\n\
+  -m, --megabytes       use 1024K-byte blocks, not 512 despite POSIXLY_CORRECT\n\
+  -s, --summarize       display only a total for each argument\n\
+  -x, --one-file-system  skip directories on different filesystems\n\
+  -D, --dereference-args  dereference PATHs when symbolic link\n\
+  -L, --dereference     dereference all symbolic links\n\
+  -S, --separate-dirs   do not include size of subdirectories\n\
+      --help            display this help and exit\n\
+      --version         output version information and exit\n"));
     }
   exit (status);
 }
-
-void
-main (argc, argv)
-     int argc;
-     char *argv[];
+
+int
+main (int argc, char **argv)
 {
   int c;
   char *cwd_only[2];
-  
+  char *bs;
+
   cwd_only[0] = ".";
   cwd_only[1] = NULL;
 
   program_name = argv[0];
-  xstat = safe_lstat;
-  output_size = getenv ("POSIXLY_CORRECT") ? size_blocks : size_kilobytes;
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
-  while ((c = getopt_long (argc, argv, "abcklsxDLS", long_options, (int *) 0))
+  xstat = lstat;
+
+  if (getenv ("POSIXLY_CORRECT"))
+    output_size = size_blocks;
+  else if ((bs = getenv ("BLOCKSIZE"))
+	   && strncmp (bs, "HUMAN", sizeof ("HUMAN") - 1) == 0)
+    {
+      opt_human_readable = 1;
+      output_size = size_bytes;
+    }
+  else
+    output_size = size_kilobytes;
+
+  while ((c = getopt_long (argc, argv, "abchklmsxDLS", long_options,
+			   (int *) 0))
 	 != EOF)
     {
       switch (c)
@@ -246,14 +281,26 @@ main (argc, argv)
 
 	case 'b':
 	  output_size = size_bytes;
+	  opt_human_readable = 0;
 	  break;
 
 	case 'c':
 	  opt_combined_arguments = 1;
 	  break;
 
+	case 'h':
+	  output_size = size_bytes;
+	  opt_human_readable = 1;
+	  break;
+
 	case 'k':
 	  output_size = size_kilobytes;
+	  opt_human_readable = 0;
+	  break;
+
+	case 'm':
+	  output_size = size_megabytes;
+	  opt_human_readable = 0;
 	  break;
 
 	case 'l':
@@ -273,7 +320,7 @@ main (argc, argv)
 	  break;
 
 	case 'L':
-	  xstat = safe_stat;
+	  xstat = stat;
 	  break;
 
 	case 'S':
@@ -287,7 +334,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("%s\n", version_string);
+      printf ("du - %s\n", PACKAGE_VERSION);
       exit (0);
     }
 
@@ -295,7 +342,7 @@ main (argc, argv)
     usage (0, NULL);
 
   if (opt_all && opt_summarize_only)
-    usage (2, "cannot both summarize and show all entries");
+    usage (2, _("cannot both summarize and show all entries"));
 
   /* Initialize the hash structure for inode numbers.  */
   hash_init (INITIAL_HASH_MODULE, INITIAL_ENTRY_TAB_SIZE);
@@ -306,36 +353,80 @@ main (argc, argv)
 
   exit (exit_status);
 }
-
+
+/* Convert N_BYTES to a more readable string than %d would.
+   Most people visually process strings of 3-4 digits effectively,
+   but longer strings of digits are more prone to misinterpretation.
+   Hence, converting to an abbreviated form usually improves readability.
+   Use a suffix indicating multiples of 1024 (K), 1024*1024 (M), and
+   1024*1024*1024 (G).  For example, 8500 would be converted to 8.3K,
+   133456345 to 127M, 56990456345 to 53G, and so on.  Numbers smaller
+   than 1024 aren't modified.  */
+
+static char *
+human_readable (int n_bytes, char *buf, int buf_len)
+{
+  const char *suffix;
+  double amt;
+  char *p;
+
+  assert (buf_len > LONGEST_HUMAN_READABLE);
+
+  p = buf;
+  amt = n_bytes;
+
+  if (amt >= 1024 * 1024 * 1024)
+    {
+      amt /= (1024 * 1024 * 1024);
+      suffix = "G";
+    }
+  else if (amt >= 1024 * 1024)
+    {
+      amt /= (1024 * 1024);
+      suffix = "M";
+    }
+  else if (amt >= 1024)
+    {
+      amt /= 1024;
+      suffix = "K";
+    }
+  else
+    {
+      suffix = "";
+    }
+
+  if (amt >= 10)
+    {
+      sprintf (p, "%.0f%s", amt, suffix);
+    }
+  else if (amt == 0)
+    {
+      strcpy (p, "0");
+    }
+  else
+    {
+      sprintf (p, "%.1f%s", amt, suffix);
+    }
+  return (p);
+}
+
 /* Recursively print the sizes of the directories (and, if selected, files)
    named in FILES, the last entry of which is NULL.  */
 
 static void
-du_files (files)
-     char **files;
+du_files (char **files)
 {
-#ifdef HAVE_FCHDIR
-  int wd_desc;
-#else
-  char *wd;
-#endif
+  struct saved_cwd cwd;
   ino_t initial_ino;		/* Initial directory's inode. */
   dev_t initial_dev;		/* Initial directory's device. */
   int i;			/* Index in FILES. */
 
-#ifdef HAVE_FCHDIR
-  wd_desc = open (".", O_RDONLY);
-  if (wd_desc < 0)
-    error (1, errno, "cannot open current directory");
-#else
-  wd = xgetcwd ();
-  if (wd == NULL)
-    error (1, errno, "cannot get current directory");
-#endif
+  if (save_cwd (&cwd))
+    exit (1);
 
   /* Remember the inode and device number of the current directory.  */
-  if (SAFE_STAT (".", &stat_buf))
-    error (1, errno, "current directory");
+  if (stat (".", &stat_buf))
+    error (1, errno, _("current directory"));
   initial_ino = stat_buf.st_ino;
   initial_dev = stat_buf.st_dev;
 
@@ -366,45 +457,46 @@ du_files (files)
       count_entry (arg, 1, 0);
 
       /* chdir if `count_entry' has changed the working directory.  */
-      if (SAFE_STAT (".", &stat_buf))
+      if (stat (".", &stat_buf))
 	error (1, errno, ".");
-      if ((stat_buf.st_ino != initial_ino || stat_buf.st_dev != initial_dev)
-#ifdef HAVE_FCHDIR
-	  && fchdir (wd_desc) < 0)
-	error (1, errno, "cannot return to original directory");
-#else
-	  && chdir (wd) < 0)
-	error (1, errno, "cannot change to directory %s", wd);
-#endif
+      if (stat_buf.st_ino != initial_ino || stat_buf.st_dev != initial_dev)
+	{
+	  if (restore_cwd (&cwd, _("starting directory"), NULL))
+	    exit (1);
+	}
     }
 
   if (opt_combined_arguments)
     {
-      printf ("%ld\ttotal\n", output_size == size_bytes ? tot_size
-	      : convert_blocks (tot_size, output_size == size_kilobytes));
+      if (opt_human_readable)
+	{
+	  char buf[LONGEST_HUMAN_READABLE + 1];
+	  printf("%s\ttotal\n", human_readable (tot_size, buf,
+						LONGEST_HUMAN_READABLE + 1));
+	}
+      else
+	{
+	  printf (_("%ld\ttotal\n"), output_size == size_bytes ? tot_size
+		  : convert_blocks (tot_size, output_size == size_kilobytes));
+	}
       fflush (stdout);
     }
 
-#ifndef HAVE_FCHDIR
-  free (wd);
-#endif
+  free_cwd (&cwd);
 }
-
+
 /* Print (if appropriate) and return the size
    (in units determined by `output_size') of file or directory ENT.
    TOP is one for external calls, zero for recursive calls.
    LAST_DEV is the device that the parent directory of ENT is on.  */
 
 static long
-count_entry (ent, top, last_dev)
-     char *ent;
-     int top;
-     dev_t last_dev;
+count_entry (char *ent, int top, dev_t last_dev)
 {
   long size;
 
   if (((top && opt_dereference_arguments)
-       ? SAFE_STAT (ent, &stat_buf)
+       ? stat (ent, &stat_buf)
        : (*xstat) (ent, &stat_buf)) < 0)
     {
       error (0, errno, "%s", path->text);
@@ -430,15 +522,31 @@ count_entry (ent, top, last_dev)
       dev_t dir_dev;
       char *name_space;
       char *namep;
+      struct saved_cwd cwd;
+      int through_symlink;
+      struct stat e_buf;
 
       dir_dev = stat_buf.st_dev;
 
       if (opt_one_file_system && !top && last_dev != dir_dev)
 	return 0;		/* Don't enter a new file system.  */
 
+#ifndef S_ISDIR
+# define S_ISDIR(s) 0
+#endif
+      /* If we're dereferencing symlinks and we're about to chdir through
+	 a symlink, remember the current directory so we can return to it
+	 later.  In other cases, chdir ("..") works fine.  */
+      through_symlink = (xstat == stat
+			 && lstat (ent, &e_buf) == 0
+			 && S_ISLNK (e_buf.st_mode));
+      if (through_symlink)
+	if (save_cwd (&cwd))
+	  exit (1);
+
       if (chdir (ent) < 0)
 	{
-	  error (0, errno, "cannot change to directory %s", path->text);
+	  error (0, errno, _("cannot change to directory %s"), path->text);
 	  exit_status = 1;
 	  return 0;
 	}
@@ -450,14 +558,20 @@ count_entry (ent, top, last_dev)
 	  if (errno)
 	    {
 	      error (0, errno, "%s", path->text);
-	      if (chdir ("..") < 0)	/* Try to return to previous dir.  */
-		error (1, errno, "cannot change to `..' from directory %s",
-		       path->text);
+	      if (through_symlink)
+		{
+		  if (restore_cwd (&cwd, "..", path->text))
+		    exit (1);
+		  free_cwd (&cwd);
+		}
+	      else if (chdir ("..") < 0)
+		  error (1, errno, _("cannot change to `..' from directory %s"),
+			 path->text);
 	      exit_status = 1;
 	      return 0;
 	    }
 	  else
-	    error (1, 0, "virtual memory exhausted");
+	    error (1, 0, _("virtual memory exhausted"));
 	}
 
       /* Remember the current path.  */
@@ -476,30 +590,62 @@ count_entry (ent, top, last_dev)
 	  namep += strlen (namep) + 1;
 	}
       free (name_space);
-      if (chdir ("..") < 0)
-        error (1, errno, "cannot change to `..' from directory %s", path->text);
+      if (through_symlink)
+	{
+	  restore_cwd (&cwd, "..", path->text);
+	  free_cwd (&cwd);
+	}
+      else if (chdir ("..") < 0)
+        error (1, errno,
+	       _("cannot change to `..' from directory %s"), path->text);
 
       str_trunc (path, pathlen - 1); /* Remove the "/" we added.  */
       if (!opt_summarize_only || top)
 	{
-	  printf ("%ld\t%s\n", output_size == size_bytes ? size
-		  : convert_blocks (size, output_size == size_kilobytes),
-		  path->length > 0 ? path->text : "/");
+	  if (opt_human_readable)
+	    {
+	      char buf[LONGEST_HUMAN_READABLE + 1];
+	      printf("%s\t%s\n",
+		     human_readable (size, buf, LONGEST_HUMAN_READABLE + 1),
+		     path->length > 0 ? path->text : "/");
+	    }
+	  else
+	    {
+	      printf ("%ld\t%s\n", (output_size == size_bytes
+				    ? size
+				    : convert_blocks (size, output_size)),
+		      path->length > 0 ? path->text : "/");
+	    }
 	  fflush (stdout);
 	}
       return opt_separate_dirs ? 0 : size;
     }
   else if (opt_all || top)
     {
-      printf ("%ld\t%s\n", output_size == size_bytes ? size
-	      : convert_blocks (size, output_size == size_kilobytes),
-	      path->text);
-      fflush (stdout);
+      /* FIXME: make this an option.  */
+      int print_only_dir_size = 0;
+      if (!print_only_dir_size)
+	{
+	  if (opt_human_readable)
+	    {
+	      char buf[LONGEST_HUMAN_READABLE + 1];
+	      printf("%s\t%s\n",
+		     human_readable (size, buf, LONGEST_HUMAN_READABLE + 1),
+		     path->length > 0 ? path->text : "/");
+	    }
+	  else
+	    {
+	      printf ("%ld\t%s\n", output_size == size_bytes ? size
+		      : convert_blocks (size, output_size == size_kilobytes),
+		      path->text);
+	    }
+	  fflush (stdout);
+	}
     }
 
   return size;
 }
-
+
 /* Allocate space for the hash structures, and set the global
    variable `htab' to point to it.  The initial hash module is specified in
    MODULUS, and the number of entries are specified in ENTRY_TAB_SIZE.  (The
@@ -508,9 +654,7 @@ count_entry (ent, top, last_dev)
    doubled.)  */
 
 static void
-hash_init (modulus, entry_tab_size)
-     unsigned modulus;
-     unsigned entry_tab_size;
+hash_init (unsigned int modulus, unsigned int entry_tab_size)
 {
   struct htab *htab_r;
 
@@ -531,7 +675,7 @@ hash_init (modulus, entry_tab_size)
    contain no entries.  */
 
 static void
-hash_reset ()
+hash_reset (void)
 {
   int i;
   struct entry **p;
@@ -545,13 +689,11 @@ hash_reset ()
 
 /* Insert an item (inode INO and device DEV) in the hash
    structure in the global variable `htab', if an entry with the same data
-   was not found already.  Return zero if the item was inserted and non-zero
+   was not found already.  Return zero if the item was inserted and nonzero
    if it wasn't.  */
 
 static int
-hash_insert (ino, dev)
-     ino_t ino;
-     dev_t dev;
+hash_insert (ino_t ino, dev_t dev)
 {
   struct htab *htab_r = htab;	/* Initially a copy of the global `htab'.  */
 
@@ -614,14 +756,11 @@ hash_insert (ino, dev)
 }
 
 /* Insert INO and DEV in the hash structure HTAB, if not
-   already present.  Return zero if inserted and non-zero if it
+   already present.  Return zero if inserted and nonzero if it
    already existed.  */
 
 static int
-hash_insert2 (htab, ino, dev)
-     struct htab *htab;
-     ino_t ino;
-     dev_t dev;
+hash_insert2 (struct htab *htab, ino_t ino, dev_t dev)
 {
   struct entry **hp, *ep2, *ep;
   hp = &htab->hash[ino % htab->modulus];
@@ -654,13 +793,11 @@ hash_insert2 (htab, ino, dev)
 
   return 0;
 }
-
+
 /* Initialize the struct string S1 for holding SIZE characters.  */
 
 static void
-str_init (s1, size)
-     string *s1;
-     unsigned size;
+str_init (string *s1, unsigned int size)
 {
   string s;
 
@@ -672,9 +809,7 @@ str_init (s1, size)
 }
 
 static void
-ensure_space (s, size)
-     string s;
-     unsigned size;
+ensure_space (string s, unsigned int size)
 {
   if (s->alloc < size)
     {
@@ -686,9 +821,7 @@ ensure_space (s, size)
 /* Assign the null-terminated C-string CSTR to S1.  */
 
 static void
-str_copyc (s1, cstr)
-     string s1;
-     char *cstr;
+str_copyc (string s1, char *cstr)
 {
   unsigned l = strlen (cstr);
   ensure_space (s1, l);
@@ -697,9 +830,7 @@ str_copyc (s1, cstr)
 }
 
 static void
-str_concatc (s1, cstr)
-     string s1;
-     char *cstr;
+str_concatc (string s1, char *cstr)
 {
   unsigned l1 = s1->length;
   unsigned l2 = strlen (cstr);
@@ -713,9 +844,7 @@ str_concatc (s1, cstr)
 /* Truncate the string S1 to have length LENGTH.  */
 
 static void
-str_trunc (s1, length)
-     string s1;
-     unsigned length;
+str_trunc (string s1, unsigned int length)
 {
   if (s1->length > length)
     {
