@@ -1,5 +1,5 @@
 /* install - copy files and set attributes
-   Copyright (C) 1989, 1990, 1991 Free Software Foundation, Inc.
+   Copyright (C) 89, 90, 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Copy files and set their permission modes and, if possible,
    their owner and group.  Used similarly to `cp'; typically
@@ -55,36 +55,44 @@
 #include <config.h>
 #include <stdio.h>
 #include <getopt.h>
-#include <ctype.h>
 #include <sys/types.h>
 #include <pwd.h>
 #include <grp.h>
+
 #include "system.h"
-#include "version.h"
-#include "safe-stat.h"
+#include "backupfile.h"
 #include "modechange.h"
 #include "makepath.h"
+#include "error.h"
+#include "xstrtol.h"
 
-#if !defined (isascii) || defined (STDC_HEADERS)
-#undef isascii
-#define isascii(c) 1
+#if HAVE_SYS_WAIT_H
+# include <sys/wait.h>
 #endif
 
-#define ISDIGIT(c) (isascii (c) && isdigit (c))
+#if HAVE_VALUES_H
+# include <values.h>
+#endif
+
+#ifndef BITSPERBYTE
+# define BITSPERBYTE 8
+#endif
 
-#ifdef _POSIX_VERSION
-#include <sys/wait.h>
-#else
 struct passwd *getpwnam ();
 struct group *getgrnam ();
+
+#ifndef _POSIX_VERSION
 uid_t getuid ();
 gid_t getgid ();
 int wait ();
 #endif
 
-#ifdef _POSIX_SOURCE
-#define endgrent()
-#define endpwent()
+#ifndef HAVE_ENDGRENT
+# define endgrent() ((void) 0)
+#endif
+
+#ifndef HAVE_ENDPWENT
+# define endpwent() ((void) 0)
 #endif
 
 /* True if C is an ASCII octal digit. */
@@ -93,22 +101,31 @@ int wait ();
 /* Number of bytes of a file to copy at a time. */
 #define READ_SIZE (32 * 1024)
 
+#ifndef UID_T_MAX
+# define UID_T_MAX ((uid_t)(~((unsigned long)1 << ((sizeof (uid_t) \
+						    * BITSPERBYTE - 1)))))
+#endif
+
+#ifndef GID_T_MAX
+# define GID_T_MAX ((gid_t)(~((unsigned long)1 << ((sizeof (gid_t) \
+						    * BITSPERBYTE - 1)))))
+#endif
+
 char *basename ();
 char *stpcpy ();
 char *xmalloc ();
-void error ();
 int safe_read ();
 int full_write ();
 int isdir ();
+enum backup_type get_version ();
 
-static int change_attributes ();
-static int copy_file ();
-static int install_file_in_dir ();
-static int install_file_in_file ();
-static int isnumber ();
-static void get_ids ();
-static void strip ();
-static void usage ();
+static int change_attributes __P ((char *path, int no_need_to_chown));
+static int copy_file __P ((char *from, char *to, int *to_created));
+static int install_file_in_dir __P ((char *from, char *to_dir));
+static int install_file_in_file __P ((char *from, char *to));
+static void get_ids __P ((void));
+static void strip __P ((char *path));
+static void usage __P ((int status));
 
 /* The name this program was run with, for error messages. */
 char *program_name;
@@ -137,10 +154,10 @@ static int strip_files;
 /* If nonzero, install a directory instead of a regular file. */
 static int dir_arg;
 
-/* If non-zero, display usage information and exit.  */
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output and exit.  */
+/* If nonzero, print the version on standard output and exit.  */
 static int show_version;
 
 static struct option const long_options[] =
@@ -150,21 +167,27 @@ static struct option const long_options[] =
   {"group", required_argument, NULL, 'g'},
   {"mode", required_argument, NULL, 'm'},
   {"owner", required_argument, NULL, 'o'},
+  {"backup", no_argument, NULL, 'b'},
+  {"version-control", required_argument, NULL, 'V'},
   {"help", no_argument, &show_help, 1},
   {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+int
+main (int argc, char **argv)
 {
   int optc;
   int errors = 0;
   char *symbolic_mode = NULL;
+  int make_backups = 0;
+  char *version;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   owner_name = NULL;
   group_name = NULL;
   mode = 0755;
@@ -172,13 +195,21 @@ main (argc, argv)
   dir_arg = 0;
   umask (0);
 
-  while ((optc = getopt_long (argc, argv, "csdg:m:o:", long_options,
+   version = getenv ("SIMPLE_BACKUP_SUFFIX");
+   if (version)
+      simple_backup_suffix = version;
+   version = getenv ("VERSION_CONTROL");
+
+  while ((optc = getopt_long (argc, argv, "bcsdg:m:o:V:S:", long_options,
 			      (int *) 0)) != EOF)
     {
       switch (optc)
 	{
 	case 0:
 	  break;
+	case 'b':
+	  make_backups = 1;
+	  break;
 	case 'c':
 	  break;
 	case 's':
@@ -196,6 +227,12 @@ main (argc, argv)
 	case 'o':
 	  owner_name = optarg;
 	  break;
+	case 'S':
+	  simple_backup_suffix = optarg;
+	  break;
+        case 'V':
+	  version = optarg;
+	  break;
 	default:
 	  usage (1);
 	}
@@ -203,7 +240,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("%s\n", version_string);
+      printf ("install - %s\n", PACKAGE_VERSION);
       exit (0);
     }
 
@@ -213,11 +250,14 @@ main (argc, argv)
   /* Check for invalid combinations of arguments. */
   if (dir_arg && strip_files)
     error (1, 0,
-	   "the strip option may not be used when installing a directory");
+	   _("the strip option may not be used when installing a directory"));
+
+  if (make_backups)
+    backup_type = get_version (version);
 
   if (optind == argc || (optind == argc - 1 && !dir_arg))
     {
-      error (0, 0, "too few arguments");
+      error (0, 0, _("too few arguments"));
       usage (1);
     }
 
@@ -225,9 +265,9 @@ main (argc, argv)
     {
       struct mode_change *change = mode_compile (symbolic_mode, 0);
       if (change == MODE_INVALID)
-	error (1, 0, "invalid mode `%s'", symbolic_mode);
+	error (1, 0, _("invalid mode `%s'"), symbolic_mode);
       else if (change == MODE_MEMORY_EXHAUSTED)
-	error (1, 0, "virtual memory exhausted");
+	error (1, 0, _("virtual memory exhausted"));
       mode = mode_adjust (0, change);
     }
 
@@ -269,9 +309,7 @@ main (argc, argv)
    Return 0 if successful, 1 if an error occurs. */
 
 static int
-install_file_in_file (from, to)
-     char *from;
-     char *to;
+install_file_in_file (char *from, char *to)
 {
   int to_created;
   int no_need_to_chown;
@@ -291,9 +329,7 @@ install_file_in_file (from, to)
    Return 0 if successful, 1 if not. */
 
 static int
-install_file_in_dir (from, to_dir)
-     char *from;
-     char *to_dir;
+install_file_in_dir (char *from, char *to_dir)
 {
   char *from_base;
   char *to;
@@ -312,15 +348,12 @@ static char buffer[READ_SIZE];
 
 /* Copy file FROM onto file TO, creating TO if necessary.
    Return 0 if the copy is successful, 1 if not.  If the copy is
-   successful, set *TO_CREATED to non-zero if TO was created (if it did
+   successful, set *TO_CREATED to nonzero if TO was created (if it did
    not exist or did, but was unlinked) and to zero otherwise.  If the
    copy fails, don't modify *TO_CREATED.  */
 
 static int
-copy_file (from, to, to_created)
-     char *from;
-     char *to;
-     int *to_created;
+copy_file (char *from, char *to, int *to_created)
 {
   int fromfd, tofd;
   int bytes;
@@ -328,29 +361,51 @@ copy_file (from, to, to_created)
   struct stat from_stats, to_stats;
   int target_created = 1;
 
-  if (SAFE_STAT (from, &from_stats))
+  if (stat (from, &from_stats))
     {
       error (0, errno, "%s", from);
       return 1;
     }
   if (!S_ISREG (from_stats.st_mode))
     {
-      error (0, 0, "`%s' is not a regular file", from);
+      error (0, 0, _("`%s' is not a regular file"), from);
       return 1;
     }
-  if (SAFE_STAT (to, &to_stats) == 0)
+  if (stat (to, &to_stats) == 0)
     {
       if (!S_ISREG (to_stats.st_mode))
 	{
-	  error (0, 0, "`%s' is not a regular file", to);
+	  error (0, 0, _("`%s' is not a regular file"), to);
 	  return 1;
 	}
       if (from_stats.st_dev == to_stats.st_dev
 	  && from_stats.st_ino == to_stats.st_ino)
 	{
-	  error (0, 0, "`%s' and `%s' are the same file", from, to);
+	  error (0, 0, _("`%s' and `%s' are the same file"), from, to);
 	  return 1;
 	}
+
+      /* The destination file exists.  Try to back it up if required.  */
+      if (backup_type != none)
+        {
+	  char *tmp_backup = find_backup_file_name (to);
+	  char *dst_backup;
+
+	  if (tmp_backup == NULL)
+	    error (1, 0, "virtual memory exhausted");
+	  dst_backup = (char *) alloca (strlen (tmp_backup) + 1);
+	  strcpy (dst_backup, tmp_backup);
+	  free (tmp_backup);
+	  if (rename (to, dst_backup))
+	    {
+	      if (errno != ENOENT)
+		{
+		  error (0, errno, "cannot backup `%s'", to);
+		  return 1;
+		}
+	    }
+	}
+
       /* If unlink fails, try to proceed anyway.  */
       if (unlink (to))
 	target_created = 0;
@@ -406,13 +461,11 @@ copy_file (from, to, to_created)
 }
 
 /* Set the attributes of file or directory PATH.
-   If NO_NEED_TO_CHOWN is non-zero, don't call chown.
+   If NO_NEED_TO_CHOWN is nonzero, don't call chown.
    Return 0 if successful, 1 if not. */
 
 static int
-change_attributes (path, no_need_to_chown)
-     char *path;
-     int no_need_to_chown;
+change_attributes (char *path, int no_need_to_chown)
 {
   int err = 0;
 
@@ -455,8 +508,7 @@ change_attributes (path, no_need_to_chown)
    it portable would be very difficult.  Not worth the effort. */
 
 static void
-strip (path)
-     char *path;
+strip (char *path)
 {
   int pid, status;
 
@@ -464,11 +516,11 @@ strip (path)
   switch (pid)
     {
     case -1:
-      error (1, errno, "cannot fork");
+      error (1, errno, _("cannot fork"));
       break;
     case 0:			/* Child. */
       execlp ("strip", "strip", path, (char *) NULL);
-      error (1, errno, "cannot run strip");
+      error (1, errno, _("cannot run strip"));
       break;
     default:			/* Parent. */
       /* Parent process. */
@@ -481,7 +533,7 @@ strip (path)
 /* Initialize the user and group ownership of the files to install. */
 
 static void
-get_ids ()
+get_ids (void)
 {
   struct passwd *pw;
   struct group *gr;
@@ -491,9 +543,11 @@ get_ids ()
       pw = getpwnam (owner_name);
       if (pw == NULL)
 	{
-	  if (!isnumber (owner_name))
-	    error (1, 0, "invalid user `%s'", owner_name);
-	  owner_id = atoi (owner_name);
+	  long int tmp_long;
+	  if (xstrtol (owner_name, NULL, 0, &tmp_long, NULL) != LONGINT_OK
+	      || tmp_long < 0 || tmp_long > UID_T_MAX)
+	    error (1, 0, _("invalid user `%s'"), owner_name);
+	  owner_id = (uid_t) tmp_long;
 	}
       else
 	owner_id = pw->pw_uid;
@@ -507,9 +561,11 @@ get_ids ()
       gr = getgrnam (group_name);
       if (gr == NULL)
 	{
-	  if (!isnumber (group_name))
-	    error (1, 0, "invalid group `%s'", group_name);
-	  group_id = atoi (group_name);
+	  long int tmp_long;
+	  if (xstrtol (group_name, NULL, 0, &tmp_long, NULL) != LONGINT_OK
+	      || tmp_long < 0 || tmp_long > (long) GID_T_MAX)
+	    error (1, 0, _("invalid group `%s'"), group_name);
+	  group_id = (gid_t) tmp_long;
 	}
       else
 	group_id = gr->gr_gid;
@@ -519,46 +575,45 @@ get_ids ()
     group_id = getgid ();
 }
 
-/* Return nonzero if STR is an ASCII representation of a nonzero
-   decimal integer, zero if not. */
-
-static int
-isnumber (str)
-     char *str;
-{
-  if (*str == 0)
-    return 0;
-  for (; *str; str++)
-    if (!ISDIGIT (*str))
-      return 0;
-  return 1;
-}
-
 static void
-usage (status)
-     int status;
+usage (int status)
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s [OPTION]... SOURCE DEST           (1st format)\n\
   or:  %s [OPTION]... SOURCE... DIRECTORY   (2nd format)\n\
-  or:  %s [OPTION]... DIRECTORY...          (3nd format)\n\
-",
+  or:  %s -d [OPTION]... DIRECTORY...       (3rd format)\n\
+"),
 	      program_name, program_name, program_name);
-      printf ("\
+      printf (_("\
+In first two formats, copy SOURCE to DEST or multiple SOURCE(s) to\n\
+DIRECTORY, while setting permission modes and owner/group.  In third\n\
+format, make all components of the given DIRECTORY(ies).\n\
 \n\
   -c                  (ignored)\n\
   -d, --directory     create [leading] directories, mandatory for 3rd format\n\
   -g, --group=GROUP   set group ownership, instead of process' current group\n\
-  -m, --mode=MODE     set permission mode (as in chmod), instead of 0755\n\
+  -m, --mode=MODE     set permission mode (as in chmod), instead of rw-r--r--\n\
   -o, --owner=OWNER   set ownership (super-user only)\n\
   -s, --strip         strip symbol tables, only for 1st and 2nd formats\n\
+  -b, --backup        make backup before removal\n\
+  -S, --suffix=SUFFIX override the usual backup suffix\n\
+  -V, --version-control=WORD   override the usual version control\n\
       --help          display this help and exit\n\
-      --version       output version information and exit\n");
+      --version       output version information and exit\n\
+\n\
+"));
+      printf (_("\
+The backup suffix is ~, unless set with SIMPLE_BACKUP_SUFFIX.  The\n\
+version control may be set with VERSION_CONTROL, values are:\n\
+\n\
+  t, numbered     make numbered backups\n\
+  nil, existing   numbered if numbered backups exist, simple otherwise\n\
+  never, simple   always make simple backups\n"));
     }
   exit (status);
 }
