@@ -1,5 +1,5 @@
 /* mkfifo -- make fifo's (named pipes)
-   Copyright (C) 1990, 1991 Free Software Foundation, Inc.
+   Copyright (C) 90, 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Options:
    -m, --mode=mode	Set the mode of created fifo's to MODE, which is
@@ -26,21 +26,18 @@
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
+
 #include "system.h"
 #include "modechange.h"
-#include "version.h"
-
-void error ();
-
-static void usage ();
+#include "error.h"
 
 /* The name this program was run with. */
 char *program_name;
 
-/* If non-zero, display usage information and exit.  */
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output and exit.  */
+/* If nonzero, print the version on standard output and exit.  */
 static int show_version;
 
 static struct option const longopts[] =
@@ -51,10 +48,29 @@ static struct option const longopts[] =
   {NULL, 0, NULL, 0}
 };
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+#ifdef S_ISFIFO
+static void
+usage (int status)
+{
+  if (status != 0)
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
+	     program_name);
+  else
+    {
+      printf (_("Usage: %s [OPTION] NAME...\n"), program_name);
+      printf (_("\
+Create named pipes (FIFOs) with the given NAMEs.\n\
+\n\
+  -m, --mode=MODE   set permission mode (as in chmod), not 0666 - umask\n\
+      --help        display this help and exit\n\
+      --version     output version information and exit\n"));
+    }
+  exit (status);
+}
+#endif
+
+int
+main (int argc, char **argv)
 {
   unsigned short newmode;
   struct mode_change *change;
@@ -63,10 +79,14 @@ main (argc, argv)
   int optc;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   symbolic_mode = NULL;
 
 #ifndef S_ISFIFO
-  error (4, 0, "fifo files not supported");
+  error (4, 0, _("fifo files not supported"));
 #else
   while ((optc = getopt_long (argc, argv, "m:", longopts, (int *) 0)) != EOF)
     {
@@ -84,7 +104,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("%s\n", version_string);
+      printf ("mkfifo - %s\n", PACKAGE_VERSION);
       exit (0);
     }
 
@@ -93,7 +113,7 @@ main (argc, argv)
 
   if (optind == argc)
     {
-      error (0, 0, "too few arguments");
+      error (0, 0, _("too few arguments"));
       usage (1);
     }
 
@@ -102,9 +122,9 @@ main (argc, argv)
     {
       change = mode_compile (symbolic_mode, 0);
       if (change == MODE_INVALID)
-	error (1, 0, "invalid mode");
+	error (1, 0, _("invalid mode"));
       else if (change == MODE_MEMORY_EXHAUSTED)
-	error (1, 0, "virtual memory exhausted");
+	error (1, 0, _("virtual memory exhausted"));
       newmode = mode_adjust (newmode, change);
     }
 
@@ -112,7 +132,7 @@ main (argc, argv)
     {
       if (mkfifo (argv[optind], newmode))
 	{
-	  error (0, errno, "cannot make fifo `%s'", argv[optind]);
+	  error (0, errno, _("cannot make fifo `%s'"), argv[optind]);
 	  errors = 1;
 	}
     }
@@ -120,24 +140,3 @@ main (argc, argv)
   exit (errors);
 #endif
 }
-
-#ifdef S_ISFIFO
-static void
-usage (status)
-     int status;
-{
-  if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
-	     program_name);
-  else
-    {
-      printf ("Usage: %s [OPTION] PATH...\n", program_name);
-      printf ("\
-\n\
-  -m, --mode=MODE   set permission mode (as in chmod), not 0666 - umask\n\
-      --help        display this help and exit\n\
-      --version     output version information and exit\n");
-    }
-  exit (status);
-}
-#endif
