@@ -1,5 +1,5 @@
 /* mknod -- make special files
-   Copyright (C) 1990, 1991 Free Software Foundation, Inc.
+   Copyright (C) 90, 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Usage: mknod [-m mode] [--mode=mode] path {bcu} major minor
 		make a block or character device node
@@ -27,35 +27,23 @@
 
    David MacKenzie <djm@ai.mit.edu>  */
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
-
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
+
 #include "system.h"
 #include "modechange.h"
-#include "version.h"
-
-void error ();
-
-static void usage ();
+#include "error.h"
+#include "xstrtol.h"
 
 /* The name this program was run with. */
 char *program_name;
 
-/* If non-zero, display usage information and exit.  */
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output and exit.  */
+/* If nonzero, print the version on standard output and exit.  */
 static int show_version;
 
 static struct option const longopts[] =
@@ -66,17 +54,47 @@ static struct option const longopts[] =
   {NULL, 0, NULL, 0}
 };
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+static void
+usage (int status)
+{
+  if (status != 0)
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
+	     program_name);
+  else
+    {
+      printf (_("Usage: %s [OPTION]... NAME TYPE [MAJOR MINOR]\n"), program_name);
+      printf (_("\
+Create the special file NAME of the given TYPE.\n\
+\n\
+  -m, --mode=MODE   set permission mode (as in chmod), not 0666 - umask\n\
+      --help        display this help and exit\n\
+      --version     output version information and exit\n\
+\n\
+MAJOR MINOR are forbidden for TYPE p, mandatory otherwise.  TYPE may be:\n\
+\n\
+  b      create a block (buffered) special file\n\
+  c, u   create a character (unbuffered) special file\n\
+  p      create a FIFO\n"));
+    }
+  exit (status);
+}
+
+int
+main (int argc, char **argv)
 {
   unsigned short newmode;
   struct mode_change *change;
   char *symbolic_mode;
   int optc;
+  int i_major, i_minor;
+  long int tmp_major, tmp_minor;
+  char *s;
 
   program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   symbolic_mode = NULL;
 
   while ((optc = getopt_long (argc, argv, "m:", longopts, (int *) 0)) != EOF)
@@ -95,7 +113,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("%s\n", version_string);
+      printf ("mknod - %s\n", PACKAGE_VERSION);
       exit (0);
     }
 
@@ -107,14 +125,24 @@ main (argc, argv)
     {
       change = mode_compile (symbolic_mode, 0);
       if (change == MODE_INVALID)
-	error (1, 0, "invalid mode");
+	error (1, 0, _("invalid mode"));
       else if (change == MODE_MEMORY_EXHAUSTED)
-	error (1, 0, "virtual memory exhausted");
+	error (1, 0, _("virtual memory exhausted"));
       newmode = mode_adjust (newmode, change);
     }
 
   if (argc - optind != 2 && argc - optind != 4)
-    usage (1);
+    {
+      const char *msg;
+      if (argc - optind < 2)
+	msg = _("too few arguments");
+      else if (argc - optind > 4)
+	msg = _("too many arguments");
+      else
+	msg = _("wrong number of arguments");
+      error (0, 0, msg);
+      usage (1);
+    }
 
   /* Only check the first character, to allow mnemonic usage like
      `mknod /dev/rst0 character 18 0'. */
@@ -123,12 +151,28 @@ main (argc, argv)
     {
     case 'b':			/* `block' or `buffered' */
 #ifndef S_IFBLK
-      error (4, 0, "block special files not supported");
+      error (4, 0, _("block special files not supported"));
 #else
       if (argc - optind != 4)
-	usage (1);
-      if (mknod (argv[optind], newmode | S_IFBLK,
-		 makedev (atoi (argv[optind + 2]), atoi (argv[optind + 3]))))
+	{
+	  error (0, 0, _("\
+when creating block special files, major and minor device\n\
+numbers must be specified"));
+	  usage (1);
+	}
+
+      s = argv[optind + 2];
+      if (xstrtol (s, NULL, 0, &tmp_major, NULL) != LONGINT_OK)
+	error (1, 0, _("invalid major device number `%s'"), s);
+
+      s = argv[optind + 3];
+      if (xstrtol (s, NULL, 0, &tmp_minor, NULL) != LONGINT_OK)
+	error (1, 0, _("invalid minor device number `%s'"), s);
+
+      i_major = (int) tmp_major;
+      i_minor = (int) tmp_minor;
+
+      if (mknod (argv[optind], newmode | S_IFBLK, makedev (i_major, i_minor)))
 	error (1, errno, "%s", argv[optind]);
 #endif
       break;
@@ -136,22 +180,42 @@ main (argc, argv)
     case 'c':			/* `character' */
     case 'u':			/* `unbuffered' */
 #ifndef S_IFCHR
-      error (4, 0, "character special files not supported");
+      error (4, 0, _("character special files not supported"));
 #else
       if (argc - optind != 4)
-	usage (1);
-      if (mknod (argv[optind], newmode | S_IFCHR,
-		 makedev (atoi (argv[optind + 2]), atoi (argv[optind + 3]))))
+	{
+	  error (0, 0, _("\
+when creating character special files, major and minor device\n\
+numbers must be specified"));
+	  usage (1);
+	}
+
+      s = argv[optind + 2];
+      if (xstrtol (s, NULL, 0, &tmp_major, NULL) != LONGINT_OK)
+	error (1, 0, _("invalid major device number `%s'"), s);
+
+      s = argv[optind + 3];
+      if (xstrtol (s, NULL, 0, &tmp_minor, NULL) != LONGINT_OK)
+	error (1, 0, _("invalid minor device number `%s'"), s);
+
+      i_major = (int) tmp_major;
+      i_minor = (int) tmp_minor;
+
+      if (mknod (argv[optind], newmode | S_IFCHR, makedev (i_major, i_minor)))
 	error (1, errno, "%s", argv[optind]);
 #endif
       break;
 
     case 'p':			/* `pipe' */
 #ifndef S_ISFIFO
-      error (4, 0, "fifo files not supported");
+      error (4, 0, _("fifo files not supported"));
 #else
       if (argc - optind != 2)
-	usage (1);
+	{
+	  error (0, 0, _("\
+major and minor device numbers may not be specified for fifo files"));
+	  usage (1);
+	}
       if (mkfifo (argv[optind], newmode))
 	error (1, errno, "%s", argv[optind]);
 #endif
@@ -163,28 +227,3 @@ main (argc, argv)
 
   exit (0);
 }
-
-static void
-usage (status)
-     int status;
-{
-  if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
-	     program_name);
-  else
-    {
-      printf ("Usage: %s [OPTION]... PATH TYPE [MAJOR MINOR]\n", program_name);
-      printf ("\
-\n\
-  -m, --mode MODE   set permission mode (as in chmod), not 0666 - umask\n\
-      --help        display this help and exit\n\
-      --version     output version information and exit\n\
-\n\
-MAJOR MINOR are forbidden for TYPE p, mandatory otherwise.  TYPE may be:\n\
-\n\
-  b      create a block (buffered) special file\n\
-  c, u   create a character (unbuffered) special file   \n\
-  p      create a FIFO\n");
-    }
-  exit (status);
-}
