@@ -1,5 +1,5 @@
 /* mv -- move or rename files
-   Copyright (C) 1986, 1989, 1990, 1991 Free Software Foundation, Inc.
+   Copyright (C) 86, 89, 90, 91, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Options:
    -f, --force		Assume a 'y' answer to all questions it would
@@ -45,10 +45,14 @@
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
+
 #include "system.h"
 #include "backupfile.h"
-#include "version.h"
-#include "safe-lstat.h"
+#include "error.h"
+
+#ifdef HAVE_LCHOWN
+# define chown(PATH, OWNER, GROUP) lchown(PATH, OWNER, GROUP)
+#endif
 
 #ifndef _POSIX_VERSION
 uid_t geteuid ();
@@ -58,18 +62,12 @@ char *basename ();
 enum backup_type get_version ();
 int isdir ();
 int yesno ();
-void error ();
 int safe_read ();
 int full_write ();
 void strip_trailing_slashes ();
-int eaccess_stat ();
+int euidaccess ();
 char *stpcpy ();
 
-static int copy_reg ();
-static int do_move ();
-static int movefile ();
-static void usage ();
-
 /* The name this program was run with. */
 char *program_name;
 
@@ -93,10 +91,13 @@ static int stdin_tty;
 /* This process's effective user ID.  */
 static uid_t myeuid;
 
-/* If non-zero, display usage information and exit.  */
+/* FIXME */
+static struct stat dest_stats, source_stats;
+
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output and exit.  */
+/* If nonzero, print the version on standard output and exit.  */
 static int show_version;
 
 static struct option const long_options[] =
@@ -113,249 +114,21 @@ static struct option const long_options[] =
   {NULL, 0, NULL, 0}
 };
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
-{
-  int c;
-  int errors;
-  int make_backups = 0;
-  char *version;
-
-  version = getenv ("SIMPLE_BACKUP_SUFFIX");
-  if (version)
-    simple_backup_suffix = version;
-  version = getenv ("VERSION_CONTROL");
-  program_name = argv[0];
-  myeuid = geteuid ();
-  interactive = override_mode = verbose = update = 0;
-  errors = 0;
-
-  while ((c = getopt_long (argc, argv, "bfiuvS:V:", long_options, (int *) 0))
-	 != EOF)
-    {
-      switch (c)
-	{
-	case 0:
-	  break;
-	case 'b':
-	  make_backups = 1;
-	  break;
-	case 'f':
-	  interactive = 0;
-	  override_mode = 1;
-	  break;
-	case 'i':
-	  interactive = 1;
-	  override_mode = 0;
-	  break;
-	case 'u':
-	  update = 1;
-	  break;
-	case 'v':
-	  verbose = 1;
-	  break;
-	case 'S':
-	  simple_backup_suffix = optarg;
-	  break;
-	case 'V':
-	  version = optarg;
-	  break;
-	default:
-	  usage (1);
-	}
-    }
-
-  if (show_version)
-    {
-      printf ("%s\n", version_string);
-      exit (0);
-    }
-
-  if (show_help)
-    usage (0);
-
-  if (argc < optind + 2)
-    {
-      error (0, 0, "missing file argument%s", argc == optind ? "s" : "");
-      usage (1);
-    }
-
-  if (make_backups)
-    backup_type = get_version (version);
-
-  stdin_tty = isatty (0);
-
-  if (argc > optind + 2 && !isdir (argv[argc - 1]))
-    error (1, 0, "when moving multiple files, last argument must be a directory");
-
-  /* Move each arg but the last onto the last. */
-  for (; optind < argc - 1; ++optind)
-    errors |= movefile (argv[optind], argv[argc - 1]);
-
-  exit (errors);
-}
-
 /* If PATH is an existing directory, return nonzero, else 0.  */
 
 static int
-is_real_dir (path)
-     char *path;
+is_real_dir (char *path)
 {
   struct stat stats;
 
-  return SAFE_LSTAT (path, &stats) == 0 && S_ISDIR (stats.st_mode);
-}
-
-/* Move file SOURCE onto DEST.  Handles the case when DEST is a directory.
-   Return 0 if successful, 1 if an error occurred.  */
-
-static int
-movefile (source, dest)
-     char *source;
-     char *dest;
-{
-  strip_trailing_slashes (source);
-
-  if ((dest[strlen (dest) - 1] == '/' && !is_real_dir (source))
-      || isdir (dest))
-    {
-      /* Target is a directory; build full target filename. */
-      char *base;
-      char *new_dest;
-
-      base = basename (source);
-      new_dest = (char *) alloca (strlen (dest) + 1 + strlen (base) + 1);
-      stpcpy (stpcpy (stpcpy (new_dest, dest), "/"), base);
-      return do_move (source, new_dest);
-    }
-  else
-    return do_move (source, dest);
-}
-
-static struct stat dest_stats, source_stats;
-
-/* Move SOURCE onto DEST.  Handles cross-filesystem moves.
-   If DEST is a directory, SOURCE must be also.
-   Return 0 if successful, 1 if an error occurred.  */
-
-static int
-do_move (source, dest)
-     char *source;
-     char *dest;
-{
-  char *dest_backup = NULL;
-
-  if (SAFE_LSTAT (source, &source_stats) != 0)
-    {
-      error (0, errno, "%s", source);
-      return 1;
-    }
-
-  if (SAFE_LSTAT (dest, &dest_stats) == 0)
-    {
-      if (source_stats.st_dev == dest_stats.st_dev
-	  && source_stats.st_ino == dest_stats.st_ino)
-	{
-	  error (0, 0, "`%s' and `%s' are the same file", source, dest);
-	  return 1;
-	}
-
-      if (S_ISDIR (dest_stats.st_mode))
-	{
-	  error (0, 0, "%s: cannot overwrite directory", dest);
-	  return 1;
-	}
-
-      if (!S_ISDIR (source_stats.st_mode) && update
-	  && source_stats.st_mtime <= dest_stats.st_mtime)
-	return 0;
-
-      if (!override_mode && (interactive || stdin_tty)
-	  && eaccess_stat (&dest_stats, W_OK, dest))
-	{
-	  fprintf (stderr, "%s: replace `%s', overriding mode %04o? ",
-		   program_name, dest,
-		   (unsigned int) (dest_stats.st_mode & 07777));
-	  if (!yesno ())
-	    return 0;
-	}
-      else if (interactive)
-	{
-	  fprintf (stderr, "%s: replace `%s'? ", program_name, dest);
-	  if (!yesno ())
-	    return 0;
-	}
-
-      if (backup_type != none)
-	{
-	  char *tmp_backup = find_backup_file_name (dest);
-	  if (tmp_backup == NULL)
-	    error (1, 0, "virtual memory exhausted");
-	  dest_backup = (char *) alloca (strlen (tmp_backup) + 1);
-	  strcpy (dest_backup, tmp_backup);
-	  free (tmp_backup);
-	  if (rename (dest, dest_backup))
-	    {
-	      if (errno != ENOENT)
-		{
-		  error (0, errno, "cannot backup `%s'", dest);
-		  return 1;
-		}
-	      else
-		dest_backup = NULL;
-	    }
-	}
-    }
-  else if (errno != ENOENT)
-    {
-      error (0, errno, "%s", dest);
-      return 1;
-    }
-
-  if (verbose)
-    printf ("%s -> %s\n", source, dest);
-
-  if (rename (source, dest) == 0)
-    {
-      return 0;
-    }
-
-  if (errno != EXDEV)
-    {
-      error (0, errno, "cannot move `%s' to `%s'", source, dest);
-      goto un_backup;
-    }
-
-  /* rename failed on cross-filesystem link.  Copy the file instead. */
-
-  if (copy_reg (source, dest))
-    goto un_backup;
-
-  if (unlink (source))
-    {
-      error (0, errno, "cannot remove `%s'", source);
-      return 1;
-    }
-
-  return 0;
-
- un_backup:
-  if (dest_backup)
-    {
-      if (rename (dest_backup, dest))
-	error (0, errno, "cannot un-backup `%s'", dest);
-    }
-  return 1;
+  return lstat (path, &stats) == 0 && S_ISDIR (stats.st_mode);
 }
 
 /* Copy regular file SOURCE onto file DEST.
    Return 1 if an error occurred, 0 if successful. */
 
 static int
-copy_reg (source, dest)
-     char *source, *dest;
+copy_reg (char *source, char *dest)
 {
   int ifd;
   int ofd;
@@ -364,14 +137,14 @@ copy_reg (source, dest)
 
   if (!S_ISREG (source_stats.st_mode))
     {
-      error (0, 0, "cannot move `%s' across filesystems: Not a regular file",
+      error (0, 0, _("cannot move `%s' across filesystems: Not a regular file"),
 	     source);
       return 1;
     }
 
   if (unlink (dest) && errno != ENOENT)
     {
-      error (0, errno, "cannot remove `%s'", dest);
+      error (0, errno, _("cannot remove `%s'"), dest);
       return 1;
     }
 
@@ -455,21 +228,160 @@ copy_reg (source, dest)
   return 0;
 }
 
+/* Move SOURCE onto DEST.  Handles cross-filesystem moves.
+   If DEST is a directory, SOURCE must be also.
+   Return 0 if successful, 1 if an error occurred.  */
+
+static int
+do_move (char *source, char *dest)
+{
+  char *dest_backup = NULL;
+
+  if (lstat (source, &source_stats) != 0)
+    {
+      error (0, errno, "%s", source);
+      return 1;
+    }
+
+  if (lstat (dest, &dest_stats) == 0)
+    {
+      if (source_stats.st_dev == dest_stats.st_dev
+	  && source_stats.st_ino == dest_stats.st_ino)
+	{
+	  error (0, 0, _("`%s' and `%s' are the same file"), source, dest);
+	  return 1;
+	}
+
+      if (S_ISDIR (dest_stats.st_mode))
+	{
+	  error (0, 0, _("%s: cannot overwrite directory"), dest);
+	  return 1;
+	}
+
+      if (!S_ISDIR (source_stats.st_mode) && update
+	  && source_stats.st_mtime <= dest_stats.st_mtime)
+	return 0;
+
+      if (!override_mode && (interactive || stdin_tty)
+	  && euidaccess (dest, W_OK))
+	{
+	  fprintf (stderr, _("%s: replace `%s', overriding mode %04o? "),
+		   program_name, dest,
+		   (unsigned int) (dest_stats.st_mode & 07777));
+	  if (!yesno ())
+	    return 0;
+	}
+      else if (interactive)
+	{
+	  fprintf (stderr, _("%s: replace `%s'? "), program_name, dest);
+	  if (!yesno ())
+	    return 0;
+	}
+
+      if (backup_type != none)
+	{
+	  char *tmp_backup = find_backup_file_name (dest);
+	  if (tmp_backup == NULL)
+	    error (1, 0, _("virtual memory exhausted"));
+	  dest_backup = (char *) alloca (strlen (tmp_backup) + 1);
+	  strcpy (dest_backup, tmp_backup);
+	  free (tmp_backup);
+	  if (rename (dest, dest_backup))
+	    {
+	      if (errno != ENOENT)
+		{
+		  error (0, errno, _("cannot backup `%s'"), dest);
+		  return 1;
+		}
+	      else
+		dest_backup = NULL;
+	    }
+	}
+    }
+  else if (errno != ENOENT)
+    {
+      error (0, errno, "%s", dest);
+      return 1;
+    }
+
+  if (verbose)
+    printf ("%s -> %s\n", source, dest);
+
+  if (rename (source, dest) == 0)
+    {
+      return 0;
+    }
+
+  if (errno != EXDEV)
+    {
+      error (0, errno, _("cannot move `%s' to `%s'"), source, dest);
+      goto un_backup;
+    }
+
+  /* rename failed on cross-filesystem link.  Copy the file instead. */
+
+  if (copy_reg (source, dest))
+    goto un_backup;
+
+  if (unlink (source))
+    {
+      error (0, errno, _("cannot remove `%s'"), source);
+      return 1;
+    }
+
+  return 0;
+
+ un_backup:
+  if (dest_backup)
+    {
+      if (rename (dest_backup, dest))
+	error (0, errno, _("cannot un-backup `%s'"), dest);
+    }
+  return 1;
+}
+
+/* Move file SOURCE onto DEST.  Handles the case when DEST is a directory.
+   Return 0 if successful, 1 if an error occurred.  */
+
+static int
+movefile (char *source, char *dest)
+{
+  strip_trailing_slashes (source);
+
+  if ((dest[strlen (dest) - 1] == '/' && !is_real_dir (source))
+      || isdir (dest))
+    {
+      /* Target is a directory; build full target filename. */
+      char *base;
+      char *new_dest;
+
+      base = basename (source);
+      /* Remove a (single) trailing slash if there is at least one.  */
+      if (dest[strlen (dest) - 1] == '/')
+        dest[strlen (dest) - 1] = '\0';
+      new_dest = (char *) alloca (strlen (dest) + 1 + strlen (base) + 1);
+      stpcpy (stpcpy (stpcpy (new_dest, dest), "/"), base);
+      return do_move (source, new_dest);
+    }
+  else
+    return do_move (source, dest);
+}
+
 static void
-usage (status)
-     int status;
+usage (int status)
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s [OPTION]... SOURCE DEST\n\
   or:  %s [OPTION]... SOURCE... DIRECTORY\n\
-",
+"),
 	      program_name, program_name);
-      printf ("\
+      printf (_("\
+Rename SOURCE to DEST, or move SOURCE(s) to DIRECTORY.\n\
 \n\
   -b, --backup                 make backup before removal\n\
   -f, --force                  remove existing destinations, never prompt\n\
@@ -481,12 +393,104 @@ Usage: %s [OPTION]... SOURCE DEST\n\
       --help                   display this help and exit\n\
       --version                output version information and exit\n\
 \n\
+"));
+      printf (_("\
 The backup suffix is ~, unless set with SIMPLE_BACKUP_SUFFIX.  The\n\
 version control may be set with VERSION_CONTROL, values are:\n\
 \n\
   t, numbered     make numbered backups\n\
   nil, existing   numbered if numbered backups exist, simple otherwise\n\
-  never, simple   always make simple backups  \n");
+  never, simple   always make simple backups\n"));
     }
   exit (status);
 }
+
+int
+main (int argc, char **argv)
+{
+  int c;
+  int errors;
+  int make_backups = 0;
+  char *version;
+
+  program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
+  version = getenv ("SIMPLE_BACKUP_SUFFIX");
+  if (version)
+    simple_backup_suffix = version;
+  version = getenv ("VERSION_CONTROL");
+
+  myeuid = geteuid ();
+  interactive = override_mode = verbose = update = 0;
+  errors = 0;
+
+  while ((c = getopt_long (argc, argv, "bfiuvS:V:", long_options, (int *) 0))
+	 != EOF)
+    {
+      switch (c)
+	{
+	case 0:
+	  break;
+	case 'b':
+	  make_backups = 1;
+	  break;
+	case 'f':
+	  interactive = 0;
+	  override_mode = 1;
+	  break;
+	case 'i':
+	  interactive = 1;
+	  override_mode = 0;
+	  break;
+	case 'u':
+	  update = 1;
+	  break;
+	case 'v':
+	  verbose = 1;
+	  break;
+	case 'S':
+	  simple_backup_suffix = optarg;
+	  break;
+	case 'V':
+	  version = optarg;
+	  break;
+	default:
+	  usage (1);
+	}
+    }
+
+  if (show_version)
+    {
+      printf ("mv - %s\n", PACKAGE_VERSION);
+      exit (0);
+    }
+
+  if (show_help)
+    usage (0);
+
+  if (argc < optind + 2)
+    {
+      error (0, 0, "%s", (argc == optind
+			  ? _("missing file arguments")
+			  : _("missing file argument")));
+      usage (1);
+    }
+
+  if (make_backups)
+    backup_type = get_version (version);
+
+  stdin_tty = isatty (STDIN_FILENO);
+
+  if (argc > optind + 2 && !isdir (argv[argc - 1]))
+    error (1, 0,
+	   _("when moving multiple files, last argument must be a directory"));
+
+  /* Move each arg but the last onto the last. */
+  for (; optind < argc - 1; ++optind)
+    errors |= movefile (argv[optind], argv[argc - 1]);
+
+  exit (errors);
+}
