@@ -1,5 +1,5 @@
 /* `rm' file deletion utility for GNU.
-   Copyright (C) 1988, 1990, 1991, 1994 Free Software Foundation, Inc.
+   Copyright (C) 88, 90, 91, 94, 95, 1996 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,76 +12,90 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-/* Written by Paul Rubin, David MacKenzie, and Richard Stallman. */
+/* Written by Paul Rubin, David MacKenzie, and Richard Stallman.  */
 
 #include <config.h>
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
+
 #include "system.h"
-#include "version.h"
-#include "safe-lstat.h"
+#include "error.h"
 
 #ifdef D_INO_IN_DIRENT
-#define D_INO(dp) ((dp)->d_ino)
+# define D_INO(dp) ((dp)->d_ino)
 #else
-/* POSIX.1 doesn't have inodes, so fake them to avoid lots of ifdefs. */
-#define D_INO(dp) 1
+/* Some systems don't have inodes, so fake them to avoid lots of ifdefs.  */
+# define D_INO(dp) 1
 #endif
 
+/* An element in a stack of pointers into `pathname'.
+   `pathp' points to where in `pathname' the terminating '\0' goes
+   for this level's directory name.  */
+struct pathstack
+{
+  struct pathstack *next;
+  char *pathp;
+  ino_t inum;
+};
+
 char *basename ();
 char *stpcpy ();
 char *xmalloc ();
 char *xrealloc ();
-int eaccess_stat ();
+int euidaccess ();
 int yesno ();
-void error ();
 void strip_trailing_slashes ();
 
-static int clear_directory ();
-static int duplicate_entry ();
-static int remove_dir ();
-static int remove_file ();
-static int rm ();
-static void usage ();
+static int clear_directory __P ((struct stat *statp));
+static int duplicate_entry __P ((struct pathstack *stack, ino_t inum));
+static int remove_dir __P ((struct stat *statp));
+static int remove_file __P ((struct stat *statp));
+static int rm __P ((void));
+static void usage __P ((int status));
 
 /* Name this program was run with.  */
 char *program_name;
 
-/* Path of file now being processed; extended as necessary. */
+/* Linked list of pathnames of directories in progress in recursive rm.
+   The entries actually contain pointers into `pathname'.
+   `pathstack' is the current deepest level.  */
+static struct pathstack *pathstack = NULL;
+
+/* Path of file now being processed; extended as necessary.  */
 static char *pathname;
 
 /* Number of bytes currently allocated for `pathname';
    made larger when necessary, but never smaller.  */
 static int pnsize;
 
-/* If nonzero, display the name of each file removed. */
+/* If nonzero, display the name of each file removed.  */
 static int verbose;
 
-/* If nonzero, ignore nonexistant files. */
+/* If nonzero, ignore nonexistant files.  */
 static int ignore_missing_files;
 
-/* If nonzero, recursively remove directories. */
+/* If nonzero, recursively remove directories.  */
 static int recursive;
 
-/* If nonzero, query the user about whether to remove each file. */
+/* If nonzero, query the user about whether to remove each file.  */
 static int interactive;
 
 /* If nonzero, remove directories with unlink instead of rmdir, and don't
    require a directory to be empty before trying to unlink it.
-   Only works for the super-user. */
+   Only works for the super-user.  */
 static int unlink_dirs;
 
-/* If nonzero, stdin is a tty. */
+/* If nonzero, stdin is a tty.  */
 static int stdin_tty;
 
-/* If non-zero, display usage information and exit.  */
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output and exit.  */
+/* If nonzero, print the version on standard output and exit.  */
 static int show_version;
 
 static struct option const long_opts[] =
@@ -96,25 +110,27 @@ static struct option const long_opts[] =
   {NULL, 0, NULL, 0}
 };
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+int
+main (int argc, char **argv)
 {
   int err = 0;
   int c;
 
+  program_name = argv[0];
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   verbose = ignore_missing_files = recursive = interactive
     = unlink_dirs = 0;
   pnsize = 256;
   pathname = xmalloc (pnsize);
-  program_name = argv[0];
 
   while ((c = getopt_long (argc, argv, "dfirvR", long_opts, (int *) 0)) != EOF)
     {
       switch (c)
 	{
-	case 0:		/* Long option. */
+	case 0:		/* Long option.  */
 	  break;
 	case 'd':
 	  unlink_dirs = 1;
@@ -141,7 +157,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("%s\n", version_string);
+      printf ("rm - %s\n", PACKAGE_VERSION);
       exit (0);
     }
 
@@ -154,12 +170,12 @@ main (argc, argv)
 	exit (0);
       else
 	{
-	  error (0, 0, "too few arguments");
+	  error (0, 0, _("too few arguments"));
 	  usage (1);
 	}
     }
 
-  stdin_tty = isatty (0);
+  stdin_tty = isatty (STDIN_FILENO);
 
   for (; optind < argc; optind++)
     {
@@ -183,10 +199,10 @@ main (argc, argv)
 }
 
 /* Remove file or directory `pathname' after checking appropriate things.
-   Return 0 if `pathname' is removed, 1 if not. */
+   Return 0 if `pathname' is removed, 1 if not.  */
 
 static int
-rm ()
+rm (void)
 {
   struct stat path_stats;
   char *base = basename (pathname);
@@ -194,11 +210,16 @@ rm ()
   if (base[0] == '.' && (base[1] == '\0'
 			     || (base[1] == '.' && base[2] == '\0')))
     {
-      error (0, 0, "cannot remove `.' or `..'");
+      error (0, 0, _("cannot remove `.' or `..'"));
       return 1;
     }
 
-  if (SAFE_LSTAT (pathname, &path_stats))
+  if (lstat (pathname, &path_stats)
+      /* The following or-clause is solely for systems like SunOS 4.1.3
+         with (broken) lstat that interpret a zero-length file name
+	 argument as something meaningful.  For such systems, manually
+	 set errno to ENOENT.  */
+      || (pathname[0] == '\0' && (errno = ENOENT)))
     {
       if (errno == ENOENT && ignore_missing_files)
 	return 0;
@@ -214,22 +235,21 @@ rm ()
 
 /* Query the user if appropriate, and if ok try to remove the
    non-directory `pathname', which STATP contains info about.
-   Return 0 if `pathname' is removed, 1 if not. */
+   Return 0 if `pathname' is removed, 1 if not.  */
 
 static int
-remove_file (statp)
-     struct stat *statp;
+remove_file (struct stat *statp)
 {
   if (!ignore_missing_files && (interactive || stdin_tty)
-      && eaccess_stat (statp, W_OK, pathname)
+      && euidaccess (pathname, W_OK)
 #ifdef S_ISLNK
       && !S_ISLNK (statp->st_mode)
 #endif
       )
     {
-      fprintf (stderr, "%s: remove %s`%s', overriding mode %04o? ",
+      fprintf (stderr, _("%s: remove %s`%s', overriding mode %04o? "),
 	       program_name,
-	       S_ISDIR (statp->st_mode) ? "directory " : "",
+	       S_ISDIR (statp->st_mode) ? _("directory ") : "",
 	       pathname,
 	       (unsigned int) (statp->st_mode & 07777));
       if (!yesno ())
@@ -237,8 +257,8 @@ remove_file (statp)
     }
   else if (interactive)
     {
-      fprintf (stderr, "%s: remove %s`%s'? ", program_name,
-	       S_ISDIR (statp->st_mode) ? "directory " : "",
+      fprintf (stderr, _("%s: remove %s`%s'? "), program_name,
+	       S_ISDIR (statp->st_mode) ? _("directory ") : "",
 	       pathname);
       if (!yesno ())
 	return 1;
@@ -258,25 +278,24 @@ remove_file (statp)
 /* If not in recursive mode, print an error message and return 1.
    Otherwise, query the user if appropriate, then try to recursively
    remove directory `pathname', which STATP contains info about.
-   Return 0 if `pathname' is removed, 1 if not. */
+   Return 0 if `pathname' is removed, 1 if not.  */
 
 static int
-remove_dir (statp)
-     struct stat *statp;
+remove_dir (struct stat *statp)
 {
   int err;
 
   if (!recursive)
     {
-      error (0, 0, "%s: is a directory", pathname);
+      error (0, 0, _("%s: is a directory"), pathname);
       return 1;
     }
 
   if (!ignore_missing_files && (interactive || stdin_tty)
-      && eaccess_stat (statp, W_OK, pathname))
+      && euidaccess (pathname, W_OK))
     {
       fprintf (stderr,
-	       "%s: descend directory `%s', overriding mode %04o? ",
+	       _("%s: descend directory `%s', overriding mode %04o? "),
 	       program_name, pathname,
 	       (unsigned int) (statp->st_mode & 07777));
       if (!yesno ())
@@ -284,7 +303,7 @@ remove_dir (statp)
     }
   else if (interactive)
     {
-      fprintf (stderr, "%s: descend directory `%s'? ",
+      fprintf (stderr, _("%s: descend directory `%s'? "),
 	       program_name, pathname);
       if (!yesno ())
 	return 1;
@@ -298,10 +317,10 @@ remove_dir (statp)
   if (interactive)
     {
       if (err)
-	fprintf (stderr, "%s: remove directory `%s' (might be nonempty)? ",
+	fprintf (stderr, _("%s: remove directory `%s' (might be nonempty)? "),
 		 program_name, pathname);
       else
-	fprintf (stderr, "%s: remove directory `%s'? ",
+	fprintf (stderr, _("%s: remove directory `%s'? "),
 		 program_name, pathname);
       if (!yesno ())
 	return 1;
@@ -315,47 +334,31 @@ remove_dir (statp)
   return 0;
 }
 
-/* An element in a stack of pointers into `pathname'.
-   `pathp' points to where in `pathname' the terminating '\0' goes
-   for this level's directory name. */
-struct pathstack
-{
-  struct pathstack *next;
-  char *pathp;
-  ino_t inum;
-};
-
-/* Linked list of pathnames of directories in progress in recursive rm.
-   The entries actually contain pointers into `pathname'.
-   `pathstack' is the current deepest level. */
-static struct pathstack *pathstack = NULL;
-
 /* Read directory `pathname' and remove all of its entries,
    avoiding use of chdir.
    On entry, STATP points to the results of stat on `pathname'.
    Return 0 for success, error count for failure.
    Upon return, `pathname' will have the same contents as before,
    but its address might be different; in that case, `pnsize' will
-   be larger, as well. */
+   be larger, as well.  */
 
 static int
-clear_directory (statp)
-     struct stat *statp;
+clear_directory (struct stat *statp)
 {
   DIR *dirp;
   struct dirent *dp;
-  char *name_space;		/* Copy of directory's filenames. */
-  char *namep;			/* Current entry in `name_space'. */
-  unsigned name_size;		/* Bytes allocated for `name_space'. */
-  int name_length;		/* Length of filename in `namep' plus '\0'. */
-  int pathname_length;		/* Length of `pathname'. */
-  ino_t *inode_space;		/* Copy of directory's inodes. */
-  ino_t *inodep;		/* Current entry in `inode_space'. */
+  char *name_space;		/* Copy of directory's filenames.  */
+  char *namep;			/* Current entry in `name_space'.  */
+  unsigned name_size;		/* Bytes allocated for `name_space'.  */
+  int name_length;		/* Length of filename in `namep' plus '\0'.  */
+  int pathname_length;		/* Length of `pathname'.  */
+  ino_t *inode_space;		/* Copy of directory's inodes.  */
+  ino_t *inodep;		/* Current entry in `inode_space'.  */
   unsigned n_inodes_allocated;	/* There is space for this many inodes
-					  in `inode_space'. */
-  int err = 0;			/* Return status. */
-  struct pathstack pathframe;	/* New top of stack. */
-  struct pathstack *pp;		/* Temporary. */
+					  in `inode_space'.  */
+  int err = 0;			/* Return status.  */
+  struct pathstack pathframe;	/* New top of stack.  */
+  struct pathstack *pp;		/* Temporary.  */
 
   name_size = statp->st_size;
   name_space = (char *) xmalloc (name_size);
@@ -384,7 +387,7 @@ clear_directory (statp)
 
       while ((dp = readdir (dirp)) != NULL)
 	{
-	  /* Skip "." and ".." (some NFS filesystems' directories lack them). */
+	  /* Skip "." and "..".  */
 	  if (dp->d_name[0] != '.'
 	      || (dp->d_name[1] != '\0'
 		  && (dp->d_name[1] != '.' || dp->d_name[2] != '\0')))
@@ -431,43 +434,43 @@ clear_directory (statp)
 	{
 	  name_length = strlen (namep) + 1;
 
-	  /* Satisfy GNU requirement that filenames can be arbitrarily long. */
+	  /* Handle arbitrarily long filenames.  */
 	  if (pathname_length + 1 + name_length > pnsize)
 	    {
 	      char *new_pathname;
 
 	      pnsize = (pathname_length + 1 + name_length) * 2;
 	      new_pathname = xrealloc (pathname, pnsize);
-	      /* Update the all the pointers in the stack to use the new area. */
+	      /* Update all pointers in the stack to use the new area.  */
 	      for (pp = pathstack; pp != NULL; pp = pp->next)
 		pp->pathp += new_pathname - pathname;
 	      pathname = new_pathname;
 	    }
 
-	  /* Add a new frame to the top of the path stack. */
+	  /* Add a new frame to the top of the path stack.  */
 	  pathframe.pathp = pathname + pathname_length;
 	  pathframe.inum = *inodep;
 	  pathframe.next = pathstack;
 	  pathstack = &pathframe;
 
-	  /* Append '/' and the filename to current pathname, take care of the
-	     file (which could result in recursive calls), and take the filename
-	     back off. */
+	  /* Append '/' and the filename to current pathname, take care of
+	     the file (which could result in recursive calls), and take
+	     the filename back off.  */
 
 	  *pathstack->pathp = '/';
 	  strcpy (pathstack->pathp + 1, namep);
 
-	  /* If the i-number has already appeared, there's an error. */
+	  /* If the i-number has already appeared, there's an error.  */
 	  if (duplicate_entry (pathstack->next, pathstack->inum))
 	    err++;
 	  else if (rm ())
 	    err++;
 
 	  *pathstack->pathp = '\0';
-	  pathstack = pathstack->next;	/* Pop the stack. */
+	  pathstack = pathstack->next;	/* Pop the stack.  */
 	}
     }
-  /* Keep trying while there are still files to remove. */
+  /* Keep trying while there are still files to remove.  */
   while (namep > name_space && err == 0);
 
   free (name_space);
@@ -480,33 +483,31 @@ clear_directory (statp)
    if yes, return 1, and if no, exit.
    This assumes that no one tries to remove filesystem mount points;
    doing so could cause duplication of i-numbers that would not indicate
-   a corrupted file system. */
+   a corrupted file system.  */
 
 static int
-duplicate_entry (stack, inum)
-     struct pathstack *stack;
-     ino_t inum;
+duplicate_entry (struct pathstack *stack, ino_t inum)
 {
-#ifndef _POSIX_SOURCE
+#ifdef D_INO_IN_DIRENT
   struct pathstack *p;
 
   for (p = stack; p != NULL; p = p->next)
     {
       if (p->inum == inum)
 	{
-	  fprintf (stderr, "\
+	  fprintf (stderr, _("\
 %s: WARNING: Circular directory structure.\n\
 This almost certainly means that you have a corrupted file system.\n\
 NOTIFY YOUR SYSTEM MANAGER.\n\
 Cycle detected:\n\
 %s\n\
-is the same file as\n", program_name, pathname);
-	  *p->pathp = '\0';	/* Truncate pathname. */
+is the same file as\n"), program_name, pathname);
+	  *p->pathp = '\0';	/* Truncate pathname.  */
 	  fprintf (stderr, "%s\n", pathname);
-	  *p->pathp = '/';	/* Put it back. */
+	  *p->pathp = '/';	/* Put it back.  */
 	  if (interactive)
 	    {
-	      fprintf (stderr, "%s: continue? ", program_name);
+	      fprintf (stderr, _("%s: continue? "), program_name);
 	      if (!yesno ())
 		exit (1);
 	      return 1;
@@ -515,21 +516,21 @@ is the same file as\n", program_name, pathname);
 	    exit (1);
 	}
     }
-#endif
+#endif /* D_INO_IN_DIRENT */
   return 0;
 }
 
 static void
-usage (status)
-     int status;
+usage (int status)
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("Usage: %s [OPTION]... PATH...\n", program_name);
-      printf ("\
+      printf (_("Usage: %s [OPTION]... FILE...\n"), program_name);
+      printf (_("\
+Remove (unlink) the FILE(s).\n\
 \n\
   -d, --directory       unlink directory, even if non-empty (super-user only)\n\
   -f, --force           ignore nonexistent files, never prompt\n\
@@ -537,7 +538,7 @@ usage (status)
   -v, --verbose         explain what is being done\n\
   -r, -R, --recursive   remove the contents of directories recursively\n\
       --help            display this help and exit\n\
-      --version         output version information and exit\n");
+      --version         output version information and exit\n"));
     }
   exit (status);
 }
